# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


standard_names = {"logL": "log_likelihood",
                  "logl": "log_likelihood",
                  "lnL": "log_likelihood",
                  "loglr": "log_likelihood",
                  "log_likelihood": "log_likelihood",
                  "tilt1": "tilt_1",
                  "tilt_1": "tilt_1",
                  "tilt_spin1": "tilt_1",
                  "theta_1l": "tilt_1",
                  "tilt2": "tilt_2",
                  "tilt_2": "tilt_2",
                  "tilt_spin2": "tilt_2",
                  "theta_2l": "tilt_2",
                  "costilt1": "cos_tilt_1",
                  "costilt2": "cos_tilt_2",
                  "redshift": "redshift",
                  "l1_optimal_snr": "L1_optimal_snr",
                  "h1_optimal_snr": "H1_optimal_snr",
                  "v1_optimal_snr": "V1_optimal_snr",
                  "L1_optimal_snr": "L1_optimal_snr",
                  "H1_optimal_snr": "H1_optimal_snr",
                  "V1_optimal_snr": "V1_optimal_snr",
                  "E1_optimal_snr": "E1_optimal_snr",
                  "optimal_snr": "network_optimal_snr",
                  "l1_matched_filter_snr": "L1_matched_filter_snr",
                  "h1_matched_filter_snr": "H1_matched_filter_snr",
                  "v1_matched_filter_snr": "V1_matched_filter_snr",
                  "L1_matched_filter_snr": "L1_matched_filter_snr",
                  "H1_matched_filter_snr": "H1_matched_filter_snr",
                  "V1_matched_filter_snr": "V1_matched_filter_snr",
                  "E1_matched_filter_snr": "E1_matched_filter_snr",
                  "matched_filter_snr": "network_matched_filter_snr",
                  "H1_cplx_snr_amp": "H1_matched_filter_abs_snr",
                  "h1_cplx_snr_amp": "H1_matched_filter_abs_snr",
                  "H1_matched_filter_snr_abs": "H1_matched_filter_abs_snr",
                  "L1_cplx_snr_amp": "L1_matched_filter_abs_snr",
                  "l1_cplx_snr_amp": "L1_matched_filter_abs_snr",
                  "L1_matched_filter_snr_abs": "L1_matched_filter_abs_snr",
                  "V1_cplx_snr_amp": "V1_matched_filter_abs_snr",
                  "v1_cplx_snr_amp": "V1_matched_filter_abs_snr",
                  "V1_matched_filter_snr_abs": "V1_matched_filter_abs_snr",
                  "E1_cplx_snr_amp": "E1_matched_filter_abs_snr",
                  "E1_matched_filter_snr_abs": "E1_matched_filter_abs_snr",
                  "H1_cplx_snr_arg": "H1_matched_filter_snr_angle",
                  "h1_cplx_snr_arg": "H1_matched_filter_snr_angle",
                  "H1_matched_filter_snr_angle": "H1_matched_filter_snr_angle",
                  "L1_cplx_snr_arg": "L1_matched_filter_snr_angle",
                  "l1_cplx_snr_arg": "L1_matched_filter_snr_angle",
                  "L1_matched_filter_snr_angle": "L1_matched_filter_snr_angle",
                  "V1_cplx_snr_arg": "V1_matched_filter_snr_angle",
                  "v1_cplx_snr_arg": "V1_matched_filter_snr_angle",
                  "V1_matched_filter_snr_angle": "V1_matched_filter_snr_angle",
                  "E1_cplx_snr_arg": "E1_matched_filter_snr_angle",
                  "E1_matched_filter_snr_angle": "E1_matched_filter_snr_angle",
                  "mc_source": "chirp_mass_source",
                  "chirpmass_source": "chirp_mass_source",
                  "chirp_mass_source": "chirp_mass_source",
                  "eta": "symmetric_mass_ratio",
                  "symmetric_mass_ratio": "symmetric_mass_ratio",
                  "m1": "mass_1",
                  "mass_1": "mass_1",
                  "mass1": "mass_1",
                  "m2": "mass_2",
                  "mass_2": "mass_2",
                  "mass2": "mass_2",
                  "ra": "ra",
                  "rightascension": "ra",
                  "dec": "dec",
                  "declination": "dec",
                  "iota": "iota",
                  "incl": "iota",
                  "cos_iota": "cos_iota",
                  "m2_source": "mass_2_source",
                  "mass_2_source": "mass_2_source",
                  "m1_source": "mass_1_source",
                  "mass_1_source": "mass_1_source",
                  "phi1": "phi_1",
                  "phi_1l": "phi_1",
                  "phi_1": "phi_1",
                  "phi2": "phi_2",
                  "phi_2l": "phi_2",
                  "phi_2": "phi_2",
                  "psi": "psi",
                  "polarisation": "psi",
                  "polarization": "psi",
                  "phi12": "phi_12",
                  "phi_12": "phi_12",
                  "phi_jl": "phi_jl",
                  "phijl": "phi_jl",
                  "a1": "a_1",
                  "a_1": "a_1",
                  "a_spin1": "a_1",
                  "spin1": "a_1",
                  "spin1_a": "a_1",
                  "a1x": "spin_1x",
                  "a1y": "spin_1y",
                  "a1z": "spin_1z",
                  "spin_1x": "spin_1x",
                  "spin_1y": "spin_1y",
                  "spin_1z": "spin_1z",
                  "a2": "a_2",
                  "a_2": "a_2",
                  "a_spin2": "a_2",
                  "spin2": "a_2",
                  "spin2_a": "a_2",
                  "a2x": "spin_2x",
                  "a2y": "spin_2y",
                  "a2z": "spin_2z",
                  "spin_2x": "spin_2x",
                  "spin_2y": "spin_2y",
                  "spin_2z": "spin_2z",
                  "chi_p": "chi_p",
                  "phase": "phase",
                  "phiorb": "phase",
                  "phi0": "phase",
                  "phiorb": "phase",
                  "phi0": "phase",
                  "distance": "luminosity_distance",
                  "dist": "luminosity_distance",
                  "luminosity_distance": "luminosity_distance",
                  "mc": "chirp_mass",
                  "chirpmass": "chirp_mass",
                  "chirp_mass": "chirp_mass",
                  "chi_eff": "chi_eff",
                  "mtotal_source": "total_mass_source",
                  "total_mass_source": "total_mass_source",
                  "mtotal": "total_mass",
                  "total_mass": "total_mass",
                  "q": "mass_ratio",
                  "mass_ratio": "mass_ratio",
                  "time": "geocent_time",
                  "tc": "geocent_time",
                  "geocent_time": "geocent_time",
                  "theta_jn": "theta_jn",
                  "cos_theta_jn": "cos_theta_jn",
                  "reference_frequency": "reference_frequency",
                  "fref": "reference_frequency",
                  "time_maxl": "marginalized_geocent_time",
                  "tref": "marginalized_geocent_time",
                  "h1_end_time": "H1_time",
                  "l1_end_time": "L1_time",
                  "v1_end_time": "V1_time",
                  "phase_maxl": "marginalized_phase",
                  "distance_maxl": "marginalized_distance",
                  "spin1_azimuthal": "a_1_azimuthal",
                  "spin1_polar": "a_1_polar",
                  "spin2_azimuthal": "a_2_azimuthal",
                  "spin2_polar": "a_2_polar",
                  "lambda1": "lambda_1",
                  "lambda_1": "lambda_1",
                  "lambda2": "lambda_2",
                  "lambda_2": "lambda_2",
                  "lambda_tilde": "lambda_tilde",
                  "lambdaT": "lambda_tilde",
                  "delta_lambda": "delta_lambda",
                  "delta_lambda_tilde": "delta_lambda",
                  "dLambdaT": "delta_lambda",
                  "logPrior": "log_prior",
                  "log_prior": "log_prior",
                  "logprior": "log_prior",
                  "weights": "weights",
                  "weight": "weights"}



#! /usr/bin/env python

# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import os

import matplotlib
matplotlib.use("Agg")
import matplotlib.pyplot as plt

from pesummary.core.plots.latex_labels import latex_labels
from pesummary.utils.utils import logger
from pesummary.core.plots import plot as core


class _PlotGeneration(object):
    """Super class to handle the plot generation for a given set of result
    files

    Parameters
    ----------
    savedir: str
        the directory to store the plots
    webdir: str
        the web directory of the run
    labels: list
        list of labels used to distinguish the result files
    samples: dict
        dictionary of posterior samples stored in the result files
    kde_plot: Bool
        if True, kde plots are generated instead of histograms, Default False
    existing_labels: list
        list of labels stored in an existing metafile
    existing_injection_data: dict
        dictionary of injection data stored in an existing metafile
    existing_samples: dict
        dictionary of posterior samples stored in an existing metafile
    same_parameters: list
        list of paramerers that are common in all result files
    injection_data: dict
        dictionary of injection data for each result file
    result_files: list
        list of result files passed
    colors: list
        colors that you wish to use to distinguish different result files
    """
    def __init__(
        self, savedir=None, webdir=None, labels=None, samples=None,
        kde_plot=False, existing_labels=None, existing_injection_data=None,
        existing_samples=None, same_parameters=None, injection_data=None,
        colors=None, custom_plotting=None, add_to_existing=False, priors={},
        include_prior=False, weights=None
    ):
        self.webdir = webdir
        self.savedir = savedir
        self.labels = labels
        self.samples = samples
        self.kde_plot = kde_plot
        self.existing_labels = existing_labels
        self.existing_injection_data = existing_injection_data
        self.existing_samples = existing_samples
        self.same_parameters = same_parameters
        self.injection_data = injection_data
        self.colors = colors
        self.custom_plotting = custom_plotting
        self.add_to_existing = add_to_existing
        self.priors = priors
        self.include_prior = include_prior
        self.weights = (
            weights if weights is not None else {i: None for i in self.labels}
        )

        if self.same_parameters is not None:
            self.same_samples = {
                param: {
                    i: self.samples[i][param] for i in self.labels
                } for param in self.same_parameters
            }
        else:
            self.same_samples = None

        for i in self.labels:
            self.check_latex_labels(self.samples[i].keys())

        self.plot_type_dictionary = {
            "corner": self.corner_plot,
            "oned_histogram": self.oned_histogram_plot,
            "sample_evolution": self.sample_evolution_plot,
            "autocorrelation": self.autocorrelation_plot,
            "oned_cdf": self.oned_cdf_plot,
            "oned_histogram_comparison": self.oned_histogram_comparison_plot,
            "oned_cdf_comparison": self.oned_cdf_comparison_plot,
            "box_plot_comparison": self.box_plot_comparison_plot,
            "custom": self.custom_plot
        }

    @staticmethod
    def check_latex_labels(parameters):
        """Check to see if there is a latex label for all parameters. If not,
        then create one

        Parameters
        ----------
        parameters: list
            list of parameters
        """
        for i in parameters:
            if i not in list(latex_labels.keys()):
                latex_labels[i] = i.replace("_", " ")

    @property
    def savedir(self):
        return self._savedir

    @savedir.setter
    def savedir(self, savedir):
        self._savedir = savedir
        if savedir is None:
            self._savedir = self.webdir + "/plots/"

    def generate_plots(self):
        """Generate all plots for all result files
        """
        for i in self.labels:
            logger.debug("Starting to generate plots for {}".format(i))
            self._generate_plots(i)
        if self.add_to_existing:
            self.add_existing_data()
        if len(self.samples) > 1:
            logger.debug("Starting to generate comparison plots")
            self._generate_comparison_plots()

    def check_prior_samples_in_dict(self, label, param):
        """Check to see if there are prior samples for a given param

        Parameters
        ----------
        label: str
            the label used to distinguish a given run
        param: str
            name of the parameter you wish to return prior samples for
        """
        cond1 = "samples" in self.priors.keys()
        if cond1 and label in self.priors["samples"].keys():
            cond1 = self.priors["samples"][label] != []
            if cond1 and param in self.priors["samples"][label].keys():
                return self.priors["samples"][label][param]
            return None
        return None

    def add_existing_data(self):
        """
        """
        from pesummary.utils.utils import _add_existing_data

        self = _add_existing_data(self)

    def _generate_plots(self, label):
        """Generate all plots for a a given result file
        """
        self.try_to_make_a_plot("corner", label=label)
        self.try_to_make_a_plot("oned_histogram", label=label)
        self.try_to_make_a_plot("sample_evolution", label=label)
        self.try_to_make_a_plot("autocorrelation", label=label)
        self.try_to_make_a_plot("oned_cdf", label=label)
        if self.custom_plotting:
            self.try_to_make_a_plot("custom", label=label)

    def _generate_comparison_plots(self):
        """Generate all comparison plots
        """
        self.try_to_make_a_plot("oned_histogram_comparison")
        self.try_to_make_a_plot("oned_cdf_comparison")
        self.try_to_make_a_plot("box_plot_comparison")

    def try_to_make_a_plot(self, plot_type, label=None):
        """Wrapper function to _try_to_make_a_plot

        Parameters
        ----------
        plot_type: str
            String to describe the plot that you wish to try and make
        label: str
            The label of the results file that you wish to plot
        """
        self._try_to_make_a_plot(
            [label], self.plot_type_dictionary[plot_type],
            "Failed to generate %s plot because {}" % (plot_type)
        )

    @staticmethod
    def _try_to_make_a_plot(arguments, function, message):
        """Try to make a plot. If it fails return an error message and continue
        plotting

        Parameters
        ----------
        arguments: list
            list of arguments that you wish to pass to function
        function: func
            function that you wish to execute
        message: str
            the error message that you wish to be printed.
        """
        try:
            function(*arguments)
        except Exception as e:
            logger.info(message.format(e))
            plt.close()

    def corner_plot(self, label):
        """Generate a corner plot for a given result file

        Parameters
        ----------
        label: str
            the label for the results file that you wish to plot
        """
        self._corner_plot(
            self.savedir, label, self.samples[label], latex_labels, self.webdir
        )

    @staticmethod
    def _corner_plot(savedir, label, samples, latex_labels, webdir):
        """Generate a corner plot for a given set of samples

        Parameters
        ----------
        savedir: str
            the directory you wish to save the plot in
        label: str
            the label corresponding to the results file
        samples: dict
            dictionary containing PESummary.utils.utils.Array objects that
            contain samples for each parameter
        latex_labels: str
            latex labels for each parameter in samples
        webdir: str
            the directory where the `js` directory is located
        """
        import warnings

        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            fig, params = core._make_corner_plot(samples, latex_labels)
            plt.savefig(
                os.path.join(
                    savedir, "corner", "{}_all_density_plots.png".format(
                        label
                    )
                )
            )
            plt.close()
            combine_corner = open(
                os.path.join(webdir, "js", "combine_corner.js")
            )
            combine_corner = combine_corner.readlines()
            params = [str(i) for i in params]
            ind = [
                linenumber for linenumber, line in enumerate(combine_corner)
                if "var list = {}" in line
            ][0]
            combine_corner.insert(
                ind + 1, "    list['{}'] = {};\n".format(label, params)
            )
            new_file = open(
                os.path.join(webdir, "js", "combine_corner.js"), "w"
            )
            new_file.writelines(combine_corner)
            new_file.close()

    def oned_histogram_plot(self, label):
        """Generate oned histogram plots for all parameters in the result file

        Parameters
        ----------
        label: str
            the label for the results file that you wish to plot
        """
        import math

        error_message = (
            "Failed to generate oned_histogram plot for %s because {}"
        )
        for param, samples in self.samples[label].items():
            if self.include_prior:
                prior = self.check_prior_samples_in_dict(label, param)
            else:
                prior = None
            arguments = [
                self.savedir, label, param, samples, latex_labels[param],
                self.injection_data[label][param], self.kde_plot, prior,
                self.weights[label]
            ]
            self._try_to_make_a_plot(
                arguments, self._oned_histogram_plot,
                error_message % (param)
            )
            continue

    def oned_histogram_comparison_plot(self, label):
        """Generate oned comparison histogram plots for all parameters that are
        common to all result files

        Parameters
        ----------
        label: str
            the label for the results file that you wish to plot
        """
        error_message = (
            "Failed to generate a comparison histogram plot for %s because {}"
        )
        for param in self.same_parameters:
            arguments = [
                self.savedir, param, self.same_samples[param],
                latex_labels[param], self.colors
            ]
            self._try_to_make_a_plot(
                arguments, self._oned_histogram_comparison_plot,
                error_message % (param)
            )
            continue

    @staticmethod
    def _oned_histogram_comparison_plot(
        savedir, parameter, samples, latex_label, colors, kde=False
    ):
        """Generate a oned comparison histogram plot for a given parameter

        Parameters
        ----------
        savedir: str
            the directory you wish to save the plot in
        parameter: str
            the name of the parameter that you wish to make a oned comparison
            histogram for
        samples: dict
            dictionary of pesummary.utils.utils.Array objects containing the
            samples that correspond to parameter for each result file. The key
            should be the corresponding label
        latex_label: str
            the latex label for parameter
        colors: list
            list of colors to be used to distinguish different result files
        kde: Bool, optional
            if True, kde plots will be generated rather than 1d histograms
        """
        same_samples = [val for key, val in samples.items()]
        fig = core._1d_comparison_histogram_plot(
            parameter, same_samples, colors, latex_label,
            list(samples.keys()), kde=kde
        )
        plt.savefig(
            os.path.join(
                savedir, "combined_1d_posterior_{}".format(parameter)
            )
        )
        plt.close()

    @staticmethod
    def _oned_histogram_plot(
        savedir, label, parameter, samples, latex_label, injection, kde=False,
        prior=None, weights=None
    ):
        """Generate a oned histogram plot for a given set of samples

        Parameters
        ----------
        savedir: str
            the directory you wish to save the plot in
        label: str
            the label corresponding to the results file
        parameter: str
            the name of the parameter that you wish to plot
        samples: PESummary.utils.utils.Array
            array containing the samples corresponding to parameter
        latex_label: str
            the latex label corresponding to parameter
        injection: float
            the injected value
        kde: Bool, optional
            if True, kde plots will be generated rather than 1d histograms
        prior: PESummary.utils.utils.Array, optional
            the prior samples for param
        weights: PESummary.utils.utils.Array, optional
            the weights for each samples. If None, assumed to be 1
        """
        import math

        if math.isnan(injection):
            injection = None

        fig = core._1d_histogram_plot(
            parameter, samples, latex_label, injection, kde=kde, prior=prior,
            weights=weights
        )
        plt.savefig(
            os.path.join(
                savedir, "{}_1d_posterior_{}.png".format(label, parameter)
            )
        )
        plt.close()

    def sample_evolution_plot(self, label):
        """Generate sample evolution plots for all parameters in the result file

        Parameters
        ----------
        label: str
            the label for the results file that you wish to plot
        """
        error_message = (
            "Failed to generate a sample evolution plot for %s because {}"
        )
        for param, samples in self.samples[label].items():
            arguments = [
                self.savedir, label, param, samples, latex_labels[param],
                self.injection_data[label][param]
            ]
            self._try_to_make_a_plot(
                arguments, self._sample_evolution_plot, error_message % (param)
            )
            continue

    @staticmethod
    def _sample_evolution_plot(
        savedir, label, parameter, samples, latex_label, injection
    ):
        """Generate a sample evolution plot for a given set of samples

        Parameters
        ----------
        savedir: str
            the directory you wish to save the plot in
        label: str
            the label corresponding to the results file
        parameter: str
            the name of the parameter that you wish to plot
        samples: PESummary.utils.utils.Array
            array containing the samples corresponding to parameter
        latex_label: str
            the latex label corresponding to parameter
        injection: float
            the injected value
        """
        fig = core._sample_evolution_plot(
            parameter, samples, latex_label, injection
        )
        plt.savefig(
            os.path.join(
                savedir, "{}_sample_evolution_{}".format(label, parameter)
            )
        )
        plt.close()

    def autocorrelation_plot(self, label):
        """Generate autocorrelation plots for all parameters in the result file

        Parameters
        ----------
        label: str
            the label for the results file that you wish to plot
        """
        error_message = (
            "Failed to generate an autocorrelation plot for %s because {}"
        )
        for param, samples in self.samples[label].items():
            arguments = [self.savedir, label, param, samples]
            self._try_to_make_a_plot(
                arguments, self._autocorrelation_plot, error_message % (param)
            )
            continue

    @staticmethod
    def _autocorrelation_plot(savedir, label, parameter, samples):
        """Generate an autocorrelation plot for a given set of samples

        Parameters
        ----------
        savedir: str
            the directory you wish to save the plot in
        label: str
            the label corresponding to the results file
        parameter: str
            the name of the parameter that you wish to plot
        samples: PESummary.utils.utils.Array
            array containing the samples corresponding to parameter
        """
        fig = core._autocorrelation_plot(parameter, samples)
        plt.savefig(
            os.path.join(
                savedir, "{}_autocorrelation_{}.png".format(
                    label, parameter
                )
            )
        )
        plt.close()

    def oned_cdf_plot(self, label):
        """Generate oned CDF plots for all parameters in the result file

        Parameters
        ----------
        label: str
            the label for the results file that you wish to plot
        """
        error_message = (
            "Failed to generate a CDF plot for %s because {}"
        )
        for param, samples in self.samples[label].items():
            arguments = [
                self.savedir, label, param, samples, latex_labels[param]
            ]
            self._try_to_make_a_plot(
                arguments, self._oned_cdf_plot, error_message % (param)
            )
            continue

    @staticmethod
    def _oned_cdf_plot(savedir, label, parameter, samples, latex_label):
        """Generate a oned CDF plot for a given set of samples

        Parameters
        ----------
        savedir: str
            the directory you wish to save the plot in
        label: str
            the label corresponding to the results file
        parameter: str
            the name of the parameter that you wish to plot
        samples: PESummary.utils.utils.Array
            array containing the samples corresponding to parameter
        latex_label: str
            the latex label corresponding to parameter
        """
        fig = core._1d_cdf_plot(parameter, samples, latex_label)
        plt.savefig(
            os.path.join(
                savedir + "{}_cdf_{}.png".format(label, parameter)
            )
        )
        plt.close()

    def oned_cdf_comparison_plot(self, label):
        """Generate oned comparison CDF plots for all parameters that are
        common to all result files

        Parameters
        ----------
        label: str
            the label for the results file that you wish to plot
        """
        error_message = (
            "Failed to generate a comparison CDF plot for %s because {}"
        )
        for param in self.same_parameters:
            arguments = [
                self.savedir, param, self.same_samples[param],
                latex_labels[param], self.colors
            ]
            self._try_to_make_a_plot(
                arguments, self._oned_cdf_comparison_plot,
                error_message % (param)
            )
            continue

    @staticmethod
    def _oned_cdf_comparison_plot(
        savedir, parameter, samples, latex_label, colors
    ):
        """Generate a oned comparison CDF plot for a given parameter

        Parameters
        ----------
        savedir: str
            the directory you wish to save the plot in
        parameter: str
            the name of the parameter that you wish to make a oned comparison
            histogram for
        samples: dict
            dictionary of pesummary.utils.utils.Array objects containing the
            samples that correspond to parameter for each result file. The key
            should be the corresponding label
        latex_label: str
            the latex label for parameter
        colors: list
            list of colors to be used to distinguish different result files
        """
        keys = list(samples.keys())
        same_samples = [samples[key] for key in keys]
        fig = core._1d_cdf_comparison_plot(
            parameter, same_samples, colors, latex_label, keys
        )
        plt.savefig(
            os.path.join(
                savedir, "combined_cdf_{}".format(parameter)
            )
        )
        plt.close()

    def box_plot_comparison_plot(self, label):
        """Generate comparison box plots for all parameters that are
        common to all result files

        Parameters
        ----------
        label: str
            the label for the results file that you wish to plot
        """
        error_message = (
            "Failed to generate a comparison box plot for %s because {}"
        )
        for param in self.same_parameters:
            arguments = [
                self.savedir, param, self.same_samples[param],
                latex_labels[param], self.colors
            ]
            self._try_to_make_a_plot(
                arguments, self._box_plot_comparison_plot,
                error_message % (param)
            )
            continue

    @staticmethod
    def _box_plot_comparison_plot(
        savedir, parameter, samples, latex_label, colors
    ):
        """Generate a comparison box plot for a given parameter

        Parameters
        ----------
        savedir: str
            the directory you wish to save the plot in
        parameter: str
            the name of the parameter that you wish to make a oned comparison
            histogram for
        samples: dict
            dictionary of pesummary.utils.utils.Array objects containing the
            samples that correspond to parameter for each result file. The key
            should be the corresponding label
        latex_label: str
            the latex label for parameter
        colors: list
            list of colors to be used to distinguish different result files
        """
        same_samples = [val for key, val in samples.items()]
        fig = core._comparison_box_plot(
            parameter, same_samples, colors, latex_label,
            list(samples.keys())
        )
        plt.savefig(
            os.path.join(savedir, "combined_boxplot_{}".format(parameter))
        )
        plt.close()

    def custom_plot(self, label):
        """Generate custom plots according to the passed python file

        Parameters
        ----------
        label: str
            the label for the results file that you wish to plot
        """
        import importlib

        if self.custom_plotting[0] != "":
            import sys

            sys.path.append(self.custom_plotting[0])
        mod = importlib.import_module(self.custom_plotting[1])

        methods = [getattr(mod, i) for i in mod.__single_plots__]
        for num, i in enumerate(methods):
            fig = i(
                list(self.samples[label].keys()), self.samples[label]
            )
            plt.savefig(
                os.path.join(
                    self.savedir, "{}_custom_plotting_{}".format(label, num)
                )
            )
            plt.close()



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import subprocess
from pathlib import Path


class GitInformation(object):
    """Helper class to handle the git information
    """
    def __init__(self):
        self.last_commit_info = self.get_last_commit_info()
        self.last_version = self.get_last_version()
        self.hash = self.last_commit_info[0]
        self.author = self.last_commit_info[1]
        self.status = self.get_status()
        self.builder = self.get_build_name()
        self.build_date = self.get_build_date()

    def call(self, arguments):
        """Launch a subprocess to run the bash command

        Parameters
        ----------
        arguments: list
            list of bash commands
        """
        return subprocess.check_output(arguments)

    def get_build_name(self):
        """Return the username and email of the current builder
        """
        try:
            name = self.call(["git", "config", "user.name"])
            email = self.call(["git", "config", "user.email"])
            name = name.strip()
            email = email.strip()
            return "%s <%s>" % (name.decode("utf-8"), email.decode("utf-8"))
        except Exception:
            return ""

    def get_build_date(self):
        """Return the current datetime
        """
        import time
        return time.strftime('%Y-%m-%d %H:%M:%S +0000', time.gmtime())

    def get_last_commit_info(self):
        """Return the details of the last git commit
        """
        try:
            string = self.call(
                ["git", "log", "-1", "--pretty=format:%h,%an,%ae"])
            string = string.decode("utf-8").split(",")
            hash, username, email = string
            author = "%s <%s>" % (username, email)
            return hash, author
        except Exception:
            return ""

    def get_status(self):
        """Return the state of the git repository
        """
        git_diff = self.call(["git", "diff", "."]).decode("utf-8")
        if git_diff:
            return "UNCLEAN: Modified working tree"
        return "CLEAN: All modifications committed"

    def get_last_version(self):
        """Return the last stable version
        """
        try:
            tag_list = self.call(["git", "tag"]).decode("utf-8").split("\n")
            tag_list = [i for i in tag_list if i.startswith('v')]
            return tag_list[-1].split("v")[-1]
        except Exception:
            return "Not found"


class PackageInformation(GitInformation):
    """Helper class to handle package versions
    """
    def __init__(self):
        self.package_info = self.get_package_info()

    def get_package_info(self):
        """Return the package information
        """
        packages = self.call(["pip", "freeze"]).decode("utf-8")
        return packages


def get_version_information():
    """Grab the version from the .version file
    """
    version_file = Path(__file__).parent / ".version"

    string = ""
    try:
        with open(version_file, "r") as f:
            f = f.readlines()
            f = [i.strip() for i in f]

        version = [i.split("= ")[1] for i in f if "last_release" in i][0]
        hash = [i.split("= ")[1] for i in f if "git_hash" in i][0]
        status = [i.split("= ")[1] for i in f if "git_status" in i][0]
        string += "%s: %s %s" % (version, status, hash)
    except IndexError:
        print("No version information found")
    except FileNotFoundError as exc:
        # if we're inside setup.py, then the file not existing is ok
        try:
            if _PESUMMARY_SETUP:
                return
        except NameError:
            pass
        raise
    return string



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import copy

import argparse
import configparser
from pesummary import conf


class ConfigAction(argparse.Action):
    """Class to extend the argparse.Action to handle dictionaries as input
    """
    def __init__(self, option_strings, dest, nargs=None, const=None,
                 default=None, type=None, choices=None, required=False,
                 help=None, metavar=None):
        super(ConfigAction, self).__init__(
            option_strings=option_strings, dest=dest, nargs=nargs,
            const=const, default=default, type=str, choices=choices,
            required=required, help=help, metavar=metavar)

    def __call__(self, parser, namespace, values, option_string=None):
        setattr(namespace, self.dest, values)

        items = {}
        config = configparser.ConfigParser()
        try:
            config.read(values)
            for key, value in config.items("pesummary"):
                if value == "True" or value == "true":
                    items[key] = True
                else:
                    if ":" in value:
                        items[key] = self.dict_from_str(value)
                    elif "," in value:
                        items[key] = self.list_from_str(value)
                    else:
                        items[key] = value
        except Exception:
            pass
        for i in vars(namespace).keys():
            if i in items.keys():
                setattr(namespace, i, items[i])

    @staticmethod
    def dict_from_str(string):
        """Reformat the string into a dictionary

        Parameters
        ----------
        string: str
            string that you would like reformatted into a dictionary
        """
        mydict = {}
        if "{" in string:
            string = string.replace("{", "")
        if "}" in string:
            string = string.replace("}", "")

        if " " in string and "," not in string:
            string = string.split(" ")
        elif "," in string and ", " not in string:
            string = string.split(",")
        elif ", " in string:
            string = string.split(", ")

        for i in string:
            value = i.split(":")
            if " " in value[0]:
                value[0] = value[0].replace(" ", "")
            if " " in value[1]:
                value[1] = value[1].replace(" ", "")
            if value[0] in mydict.keys():
                mydict[value[0]].append(value[1])
            else:
                mydict[value[0]] = [value[1]]
        return mydict

    @staticmethod
    def list_from_str(string):
        """Reformat the string into a list

        Parameters
        ----------
        string: str
            string that you would like reformatted into a list
        """
        list = []
        if "[" in string:
            string.replace("[", "")
        if "]" in string:
            string.replace("]", "")
        if ", " in string:
            list = string.split(", ")
        elif " " in string:
            list = string.split(" ")
        elif "," in string:
            list = string.split(",")
        return list


def command_line():
    """Generate an Argument Parser object to control the command line options
    """
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("pesummary", nargs='?', help=(
                        "configuration file containing the command line "
                        "arguments"), action=ConfigAction)
    parser.add_argument("-w", "--webdir", dest="webdir",
                        help="make page and plots in DIR", metavar="DIR",
                        default=None)
    parser.add_argument("-b", "--baseurl", dest="baseurl",
                        help="make the page at this url", metavar="DIR",
                        default=None)
    parser.add_argument("-s", "--samples", dest="samples",
                        help="Posterior samples hdf5 file", nargs='+',
                        default=None)
    parser.add_argument("-c", "--config", dest="config",
                        help=("configuration file associcated with "
                              "each samples file."),
                        nargs='+', default=None)
    parser.add_argument("--email", action="store",
                        help=("send an e-mail to the given address with a link "
                              "to the finished page."), default=None)
    parser.add_argument("--dump", action="store_true",
                        help="dump all information onto a single html page",
                        default=False)
    parser.add_argument("--add_to_existing", action="store_true",
                        help="add new results to an existing html page",
                        default=False)
    parser.add_argument("-e", "--existing_webdir", dest="existing",
                        help="web directory of existing output", default=None)
    parser.add_argument("-i", "--inj_file", dest="inj_file",
                        help="path to injetcion file", nargs='+', default=None)
    parser.add_argument("--user", dest="user", help=argparse.SUPPRESS,
                        default=conf.user)
    parser.add_argument("--labels", dest="labels",
                        help="labels used to distinguish runs", nargs='+',
                        default=None)
    parser.add_argument("--compare_results", dest="compare_results",
                        help="labels for events stored in the "
                             "posterior_samples.json that you wish to compare",
                        nargs='+', default=None)
    parser.add_argument("-v", "--verbose", action="store_true",
                        help="print useful information for debugging purposes")
    parser.add_argument("--save_to_hdf5", action="store_true",
                        help="save the meta file in hdf5 format", default=False)
    parser.add_argument("--custom_plotting", dest="custom_plotting",
                        help=("python file containing functions for custom "
                              "plotting"), default=None)
    parser.add_argument("--publication", action="store_true",
                        help="generate production quality plots", default=None)
    parser.add_argument("--kde_plot", action="store_true",
                        help="plot a kde rather than a histogram",
                        default=False)
    parser.add_argument("--palette", dest="palette",
                        help="Color palette to use to distinguish result files",
                        default="colorblind")
    parser.add_argument("--burnin", dest="burnin",
                        help="Number of samples to remove as burnin",
                        default=None)
    parser.add_argument("--include_prior", action="store_true",
                        help="Plot the prior on the same plot as the posterior",
                        default=False)
    parser.add_argument("--notes", dest="notes",
                        help=("Single file containing notes that you wish to "
                              "put on summarypages"), default=None)
    parser.add_argument("--prior_file", dest="prior_file",
                        help="Prior samples file", nargs='+', default=None)
    return parser



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from pesummary.core.webpage.base import Base


class table_of_images(Base):

    def __init__(self, content, rows, columns, html_file, code, cli):
        """

        Parameters
        ----------
        content: nd list
            nd list containing the image paths that you want to include in your
            table. Sorted by columns [[column1], [column2]]
        """
        self.content = content
        self.rows = rows
        self.columns = columns
        self.html_file = html_file
        self.code = code
        self.cli = cli
        self._add_scripts()

    def _add_scripts(self):
        self.add_content("<link rel='stylesheet' href='../css/image_styles.css'>\n")

    def _insert_image(self, path, width, indent, _id, justify="center"):
        string = "<img src='{}' alt='No image available' ".format(path) + \
                 "style='align-items:center; width:{}px;'".format(width) + \
                 "id={} onclick='{}(\"{}\")'>\n".format(_id, self.code, _id)
        self.add_content(string, indent=indent)

    def make(self):
        self.make_container()
        self.make_div(2, _class="mx-auto d-block", _style=None)
        if self.rows == 1:
            _id = self.content[0][0].split("/")[-1][:-4]
            self._insert_image(self.content[0][0], 750, 4, _id, justify="left")
            self.make_div(2, _class=None, _style="float: right;")
            for i in self.content[1]:
                _id = i.split("/")[-1][:-4]
                self.make_div(2, _class="row")
                self.make_div(4, _class="column")
                self.add_content("<a>", 6)
                self._insert_image(i, 375, 8, _id, justify=None)
                self.add_content("</a>", 6)
                self.end_div(4)
                self.end_div(2)
            self.end_div(2)
        else:
            ind = 0
            width = "280"
            _class = "row justify-content-center"
            self.make_div(4, _class=_class, _style=None)
            self.make_div(6, _class="row", _style=None)
            for idx, i in enumerate(self.content):
                self.make_div(8, _class="column", _style="padding-left: 1em;")
                for num, j in enumerate(i):
                    _id = j.split("/")[-1][:-4]
                    self.make_div(10, _class='container',
                                  _style=("display: inline-block; width: auto; "
                                          "padding: 0;"))
                    self.add_content(
                        "<a href='#demo' data-slide-to='%s'>\n" % (ind), indent=6)
                    self._insert_image(j, width, 8, _id, justify=None)
                    self.add_content("</a>\n", indent=6)
                    if self.cli:
                        self.make_div(10, _class="imgButton", _style=None)
                        self.add_content("<button value='test' data-toggle='modal' "
                                         "data-target='#my%s%sModal'>Command Line<"
                                         "/button>" % (idx, num), indent=12)
                        self.end_div(10)
                        self.add_content("<div class='modal' id='my%s%sModal'>" % (
                            idx, num), indent=10)
                        self.make_div(12, _class='modal-dialog',
                                      _style="max-width: 1000px; padding-top: 250px;")
                        self.make_div(14, _class='modal-content', _style=None)
                        self.make_div(16, _class='modal-body',
                                      _style="font-size: 0.75rem;")
                        self.add_content("%s" % (self.cli[idx][num]), indent=18)
                        self.end_div(16)
                        self.end_div(14)
                        self.end_div(12)
                        self.end_div(10)
                    self.end_div(10)
                    ind += 1
                self.end_div(8)
            self.end_div(6)
            self.end_div(4)
        self.end_div(2)
        self.end_container()



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import numpy as np


class Base(object):
    """Meta class containing helper functions for generating webpages
    """
    def close(self):
        """Close the opened html file.
        """
        self.html_file.close()

    def _check_content(self, content):
        """Make sure that the content has new line in string

        Parameters
        ----------
        content: str
            string that you want to check
        """
        if content[-1] != "\n":
            content += "\n"
        return content

    def add_content(self, content, indent=0):
        """Add content to the html page

        Parameters
        ----------
        content: str/list, optional
            either a single string or a list of string that you want to add to
            your html page
        indent: int, optional
            the indent of the line
        """
        if type(content) == list:
            for i in np.arange(len(content)):
                content[i] == self._check_content(content[i])
                self.html_file.write(" " * indent + content)
        else:
            content = self._check_content(content)
            self.html_file.write(" " * indent + content)

    def make_div(self, indent=0, _class=None, _style=None, _id=None):
        """Make a div of your choice

        indent: int, optional
            the indent of the line
        _class: str, optional
            the class name of your div
        _style: str, optional
            the style of your div
        _id: str, optional
            the id of your div
        """
        string = "<div"
        if _class:
            string += " class='%s'" % (_class)
        if _style:
            string += " style='%s'" % (_style)
        if _id:
            string += " id='%s'" % (_id)
        string += ">\n"
        self.add_content(string, indent=indent)

    def end_div(self, indent=0):
        """End a div of your choice

        Parameters
        ----------
        indent: int, optional
            the indent of the new line
        """
        self.add_content("</div>", indent)

    def make_container(self, style=None, indent=0):
        """Make a container for your webpage

        Parameters
        ----------
        indent: int, optional
            the indent of the new line
        """
        if not style:
            style = "margin-top:3em; margin-bottom:5em; background-color:#FFFFFF; " + \
                    "box-shadow: 0 0 5px grey;"
        self.make_div(indent, _class="container", _style=style)

    def end_container(self, indent=0):
        """End a container

         Parameters
         ----------
         indent: int, optional
             the indent of the new line
         """
        self.end_div(indent)



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org> This program is free
# software; you can redistribute it and/or modify it under the terms of the GNU
# General Public License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

try:
    from ligo.computeDiskMass import computeDiskMass
    PASTRO = True
except ImportError:
    PASTRO = False

import numpy as np
from pesummary.utils.utils import logger


def get_probabilities(samples):
    """Return `HasNS` and `HasRemnant` probabilities from a dictionary of
    samples

    Parameters
    ----------
    samples: dict
        dictionary of samples
    """
    from pesummary.utils.utils import RedirectLogger

    default_error = (
        "Failed to generate `HasNS` and `HasRemnant` probabilities because {}"
    )
    try:
        with RedirectLogger("p_astro", level="DEBUG") as redirector:
            data = PAstro.classifications(samples)
    except ImportError:
        logger.warn(default_error.format("'p_astro' is not installed"))
        data = None
    except Exception as e:
        logger.warn(default_error.format("%s" % (e)))
        data = None
    return data


class PAstro(object):
    """Class to handle the p_astro package
    """
    @staticmethod
    def check_for_install():
        """Check that p_astro is installed
        """
        if not PASTRO:
            raise ImportError(
                "Failed to import 'p_astro' packages and therefore unable to "
                "calculate `HasNS` and `HasRemnant` probabilities"
            )

    @staticmethod
    def classifications(samples):
        """Calculate the `HasNS` and `HasRemnant` probabilities

        Parameters
        ----------
        samples: dict
            dictionary of samples
        """
        PAstro.check_for_install()
        required_params = ["mass_1", "mass_2", "a_1", "a_2"]
        parameters = list(samples.keys())

        if not all(i in parameters for i in required_params):
            raise Exception(
                "Failed to generate `HasNS` and `HasRemnant` probabilities "
                "because not all required parameters have been provided."
            )
        M_rem = computeDiskMass(
            samples["mass_1"], samples["mass_2"], samples["a_1"], samples["a_2"]
        )
        prediction_ns = float(
            np.sum(samples["mass_2"] <= 3.0) / len(samples["mass_2"])
        )
        prediction_em = float(np.sum(M_rem > 0) / len(M_rem))
        return {"HasNS": prediction_ns, "HasRemnant": prediction_em}



#! /usr/bin/env python

# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import os
import pesummary
from pesummary.gw.file.read import read as GWRead
from pesummary.gw.pepredicates import PEPredicates
from pesummary.gw.p_astro import PAstro
from pesummary.utils.utils import make_dir, logger
from pesummary.utils.exceptions import InputError
import argparse


__doc__ = """This executable is used to generate a txt file containing the
source classification probailities"""


def command_line():
    """Generate an Argument Parser object to control the command line options
    """
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("-w", "--webdir", dest="webdir",
                        help="make page and plots in DIR", metavar="DIR",
                        default=None)
    parser.add_argument("-s", "--samples", dest="samples",
                        help="Posterior samples hdf5 file", nargs='+',
                        default=None)
    parser.add_argument("--labels", dest="labels",
                        help="labels used to distinguish runs", nargs='+',
                        default=None)
    parser.add_argument("--plot_with_population_prior", action="store_true",
                        help=("generate PEPredicates plots for samples reweighted "
                              "to a population prior"), default=False)
    parser.add_argument("--plot_with_default_prior", action="store_true",
                        help="generate PEPredicates plots for samples",
                        default=False)
    parser.add_argument("--plot", dest="plot",
                        help="name of the plot you wish to make",
                        default="bar", choices=["bar", "mass_1_mass_2"])
    return parser


def generate_probabilities(result_files):
    """Generate the classification probabilities

    Parameters
    ----------
    result_files: list
        list of result files
    """
    classifications = []

    for num, i in enumerate(result_files):
        mydict = {}
        f = GWRead(i)
        if not isinstance(f, pesummary.gw.file.formats.pesummary.PESummary):
            f.generate_all_posterior_samples()
        mydict["default"], mydict["population"] = \
            PEPredicates.classifications(f.samples, f.parameters)
        em_bright = PAstro.classifications(f.samples_dict)
        mydict["default"]["HasNS"] = em_bright["HasNS"]
        mydict["default"]["HasRemnant"] = em_bright["HasRemnant"]
        classifications.append(mydict)
    return classifications


def save_classifications(savedir, classifications, labels):
    """Read and return a list of parameters and samples stored in the result
    files

    Parameters
    ----------
    result_files: list
        list of result files
    classifications: dict
        dictionary of classification probabilities
    """
    import os
    import json

    if labels is None:
        raise InputError("Please provide a label for each result file")
    base_path = os.path.join(savedir, "{}_{}_prior_pe_classification.json")
    for num, i in enumerate(classifications):
        with open(base_path.format(labels[num], "default"), "w") as f:
            json.dump(i["default"], f)
        with open(base_path.format(labels[num], "population"), "w") as f:
            json.dump(i["population"], f)


def make_plots(
    result_files, webdir=None, labels=None, prior=None, plot_type="bar",
    probs=None
):
    """Save the plots generated by PEPredicates

    Parameters
    ----------
    result_files: list
        list of result files
    webdir: str
        path to save the files
    labels: list
        lisy of strings to identify each result file
    prior: str
        Either 'default' or 'population'. If 'population' the samples are reweighted
        to a population prior
    plot_type: str
        The plot type that you wish to make
    probs: dict
        Dictionary of classification probabilities
    """
    import matplotlib.pyplot as plt

    if webdir is None:
        webdir = "./"

    for num, i in enumerate(result_files):
        if labels is None:
            label = num
        else:
            label = labels[num]
        f = GWRead(i)
        if not isinstance(f, pesummary.gw.file.formats.pesummary.PESummary):
            f.generate_all_posterior_samples()
        if plot_type == "bar":
            from pesummary.gw.plots.plot import _classification_plot

            if prior == "default":
                fig = _classification_plot(probs[num]["default"])
                plt.savefig(
                    os.path.join(
                        webdir,
                        "{}_default_pepredicates_bar.png".format(label)
                    )
                )
            else:
                fig = _classification_plot(probs[num]["population"])
                plt.savefig(
                    os.path.join(
                        webdir,
                        "{}_population_pepredicates_bar.png".format(label)
                    )
                )
        elif plot_type == "mass_1_mass_2":
            if prior == "default":
                fig = PEPredicates.plot(
                    f.samples, f.parameters, population_prior=False
                )
                plt.savefig(
                    os.path.join(
                        webdir, "{}_default_pepredicates.png".format(label)
                    )
                )
            else:
                fig = PEPredicates.plot(f.samples, f.parameters)
                plt.savefig(
                    os.path.join(
                        webdir, "{}_population_pepredicates.png".format(label)
                    )
                )


def main():
    """Top level interface for `summarypublication`
    """
    parser = command_line()
    opts = parser.parse_args()
    if opts.webdir:
        make_dir(opts.webdir)
    classifications = generate_probabilities(
        opts.samples)
    if opts.webdir:
        save_classifications(opts.webdir, classifications, opts.labels)
    if opts.plot_with_default_prior:
        prior = "default"
    else:
        prior = "population"
    if opts.plot == "bar":
        probs = classifications
    else:
        probs = None
    make_plots(
        opts.samples, webdir=opts.webdir, labels=opts.labels, prior=prior,
        plot_type=opts.plot, probs=probs
    )


if __name__ == "__main__":
    main()



#! /usr/bin/env python

# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import os

import matplotlib
matplotlib.use("Agg")
import matplotlib.pyplot as plt
import multiprocessing as mp
import numpy as np

from pesummary.core.plots.main import _PlotGeneration as _BasePlotGeneration
from pesummary.core.plots.latex_labels import latex_labels
from pesummary.gw.plots.latex_labels import GWlatex_labels
from pesummary.utils.utils import logger, resample_posterior_distribution
from pesummary.gw.plots import publication
from pesummary.gw.plots import plot as gw
from pesummary import conf


latex_labels.update(GWlatex_labels)


class _PlotGeneration(_BasePlotGeneration):
    def __init__(
        self, savedir=None, webdir=None, labels=None, samples=None,
        kde_plot=False, existing_labels=None, existing_injection_data=None,
        existing_file_kwargs=None, existing_samples=None,
        existing_metafile=None, same_parameters=None, injection_data=None,
        result_files=None, file_kwargs=None, colors=None, custom_plotting=None,
        add_to_existing=False, priors={}, no_ligo_skymap=False,
        nsamples_for_skymap=None, detectors=None, maxL_samples=None,
        gwdata=None, calibration=None, psd=None,
        multi_threading_for_skymap=None, approximant=None,
        pepredicates_probs=None, include_prior=False, publication=False,
        existing_approximant=None, existing_psd=None, existing_calibration=None,
        weights=None
    ):
        super(_PlotGeneration, self).__init__(
            savedir=savedir, webdir=webdir, labels=labels,
            samples=samples, kde_plot=kde_plot, existing_labels=existing_labels,
            existing_injection_data=existing_injection_data,
            existing_samples=existing_samples,
            same_parameters=same_parameters,
            injection_data=injection_data,
            colors=colors, custom_plotting=custom_plotting,
            add_to_existing=add_to_existing, priors=priors,
            include_prior=include_prior, weights=weights
        )
        self.file_kwargs = file_kwargs
        self.existing_file_kwargs = existing_file_kwargs
        self.no_ligo_skymap = no_ligo_skymap
        self.nsamples_for_skymap = nsamples_for_skymap
        self.detectors = detectors
        self.maxL_samples = maxL_samples
        self.gwdata = gwdata
        self.calibration = calibration
        self.existing_calibration = existing_calibration
        self.psd = psd
        self.existing_psd = existing_psd
        self.multi_threading_for_skymap = multi_threading_for_skymap
        self.approximant = approximant
        self.existing_approximant = existing_approximant
        self.pepredicates_probs = pepredicates_probs
        self.publication = publication

        self.plot_type_dictionary = {
            "corner": self.corner_plot,
            "oned_histogram": self.oned_histogram_plot,
            "sample_evolution": self.sample_evolution_plot,
            "autocorrelation": self.autocorrelation_plot,
            "oned_cdf": self.oned_cdf_plot,
            "oned_histogram_comparison": self.oned_histogram_comparison_plot,
            "oned_cdf_comparison": self.oned_cdf_comparison_plot,
            "box_plot_comparison": self.box_plot_comparison_plot,
            "custom": self.custom_plot,
            "psd": self.psd_plot,
            "calibration": self.calibration_plot,
            "skymap": self.skymap_plot,
            "waveform_fd": self.waveform_fd_plot,
            "waveform_td": self.waveform_td_plot,
            "data": self.strain_plot,
            "skymap_comparison": self.skymap_comparison_plot,
            "waveform_comparison_fd": self.waveform_comparison_fd_plot,
            "waveform_comparison_td": self.waveform_comparison_td_plot,
            "2d_comparison_contour": self.twod_comparison_contour_plot,
            "violin": self.violin_plot,
            "spin_disk": self.spin_dist_plot,
            "pepredicates": self.pepredicates_plot
        }

    def generate_plots(self):
        """Generate all plots for all result files
        """
        for i in self.labels:
            logger.debug("Starting to generate plots for {}".format(i))
            self._generate_plots(i)
        if self.calibration or "calibration" in list(self.priors.keys()):
            self.try_to_make_a_plot("calibration")
        if self.psd:
            self.try_to_make_a_plot("psd")
        if self.add_to_existing:
            self.add_existing_data()
        if len(self.samples) > 1:
            logger.debug("Starting to generate comparison plots")
            self._generate_comparison_plots()

    def _generate_plots(self, label):
        """Generate all plots for a given result file
        """
        self.try_to_make_a_plot("corner", label=label)
        self.try_to_make_a_plot("skymap", label=label)
        self.try_to_make_a_plot("waveform_td", label=label)
        self.try_to_make_a_plot("waveform_fd", label=label)
        if self.pepredicates_probs[label] is not None:
            self.try_to_make_a_plot("pepredicates", label=label)
        if self.gwdata:
            self.try_to_make_a_plot("data", label=label)
        self.try_to_make_a_plot("oned_histogram", label=label)
        self.try_to_make_a_plot("sample_evolution", label=label)
        self.try_to_make_a_plot("autocorrelation", label=label)
        self.try_to_make_a_plot("oned_cdf", label=label)
        if self.custom_plotting:
            self.try_to_make_a_plot("custom", label=label)

    def _generate_comparison_plots(self):
        """Generate all comparison plots
        """
        self.try_to_make_a_plot("skymap_comparison")
        self.try_to_make_a_plot("waveform_comparison_td")
        self.try_to_make_a_plot("waveform_comparison_fd")
        self.try_to_make_a_plot("oned_histogram_comparison")
        self.try_to_make_a_plot("oned_cdf_comparison")
        self.try_to_make_a_plot("box_plot_comparison")
        if self.publication:
            self.try_to_make_a_plot("2d_comparison_contour")
            self.try_to_make_a_plot("violin")
            self.try_to_make_a_plot("spin_disk")

    @staticmethod
    def _corner_plot(savedir, label, samples, latex_labels, webdir):
        """Generate a corner plot for a given set of samples

        Parameters
        ----------
        savedir: str
            the directory you wish to save the plot in
        label: str
            the label corresponding to the results file
        samples: dict
            dictionary of samples for a given result file
        latex_labels: dict
            dictionary of latex labels
        webdir: str
            directory where the javascript is written
        """
        import warnings

        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            fig, params = gw._make_corner_plot(samples, latex_labels)
            plt.savefig(
                os.path.join(
                    savedir, "corner", "{}_all_density_plots.png".format(
                        label
                    )
                )
            )
            plt.close()
            combine_corner = open(
                os.path.join(webdir, "js", "combine_corner.js")
            )
            combine_corner = combine_corner.readlines()
            params = [str(i) for i in params]
            ind = [
                linenumber for linenumber, line in enumerate(combine_corner)
                if "var list = {}" in line
            ][0]
            combine_corner.insert(
                ind + 1, "    list['{}'] = {};\n".format(label, params)
            )
            new_file = open(
                os.path.join(webdir, "js", "combine_corner.js"), "w"
            )
            new_file.writelines(combine_corner)
            new_file.close()
            fig = gw._make_source_corner_plot(samples, latex_labels)
            plt.savefig(
                os.path.join(
                    savedir, "corner", "{}_sourceframe.png".format(label)
                )
            )
            plt.close()
            fig = gw._make_extrinsic_corner_plot(samples, latex_labels)
            plt.savefig(
                os.path.join(
                    savedir, "corner", "{}_extrinsic.png".format(label)
                )
            )
            plt.close()

    def skymap_plot(self, label):
        """Generate a skymap plot for a given result file

        Parameters
        ----------
        label: str
            the label for the results file that you wish to plot
        """
        try:
            import ligo.skymap
            SKYMAP = True
        except ImportError:
            SKYMAP = False

        self._skymap_plot(
            self.savedir, self.samples[label]["ra"], self.samples[label]["dec"],
            label, self.weights[label]
        )

        if SKYMAP and not self.no_ligo_skymap:
            from pesummary.utils.utils import RedirectLogger

            logger.info("Launching subprocess to generate skymap plot with "
                        "ligo.skymap")
            with RedirectLogger("ligo.skymap", level="DEBUG") as redirector:
                process = mp.Process(
                    target=self._ligo_skymap_plot,
                    args=[
                        self.savedir, self.samples[label]["ra"],
                        self.samples[label]["dec"], label,
                        self.nsamples_for_skymap, self.webdir,
                        self.multi_threading_for_skymap
                    ]
                )
                process.start()

    @staticmethod
    def _skymap_plot(savedir, ra, dec, label, weights):
        """Generate a skymap plot for a given set of samples

        Parameters
        ----------
        savedir: str
            the directory you wish to save the plot in
        ra: pesummary.utils.utils.Array
            array containing the samples for right ascension
        dec: pesummary.utils.utils.Array
            array containing the samples for declination
        label: str
            the label corresponding to the results file
        weights: list
            list of weights for the samples
        """
        fig = gw._default_skymap_plot(ra, dec, weights)
        plt.savefig(
            os.path.join(savedir, "{}_skymap.png".format(label))
        )
        plt.close()

    @staticmethod
    def _ligo_skymap_plot(savedir, ra, dec, label, nsamples_for_skymap,
                          webdir, multi_threading_for_skymap):
        """Generate a skymap plot for a given set of samples using the
        ligo.skymap package

        Parameters
        ----------
        savedir: str
            the directory you wish to save the plot in
        ra: pesummary.utils.utils.Array
            array containing the samples for right ascension
        dec: pesummary.utils.utils.Array
            array containing the samples for declination
        label: str
            the label corresponding to the results file
        nsamples_for_skymap: int
            the number of samples used to generate skymap
        webdir: str
            the directory to store the fits file
        """
        downsampled = False
        if nsamples_for_skymap is not None:
            ra, dec = resample_posterior_distribution(
                [ra, dec], nsamples_for_skymap
            )
            downsampled = True
        fig = gw._ligo_skymap_plot(
            ra, dec, savedir=os.path.join(webdir, "samples"),
            nprocess=multi_threading_for_skymap, downsamples=downsampled
        )
        plt.savefig(
            os.path.join(savedir, "{}_skymap.png".format(label))
        )
        plt.close()

    def waveform_fd_plot(self, label):
        """Generate a frequency domain waveform plot for a given result file

        Parameters
        ----------
        label: str
            the label corresponding to the results file
        """
        if self.approximant[label] == {}:
            return
        self._waveform_fd_plot(
            self.savedir, self.detectors[label], self.maxL_samples[label], label
        )

    @staticmethod
    def _waveform_fd_plot(savedir, detectors, maxL_samples, label):
        """Generate a frequency domain waveform plot for a given detector
        network and set of samples

        Parameters
        ----------
        savedir: str
            the directory you wish to save the plot in
        detectors: list
            list of detectors used in your analysis
        maxL_samples: dict
            dictionary of maximum likelihood values
        label: str
            the label corresponding to the results file
        """
        if detectors is None:
            detectors = ["H1", "L1"]
        else:
            detectors = detectors.split("_")

        fig = gw._waveform_plot(detectors, maxL_samples)
        plt.savefig(
            os.path.join(savedir, "{}_waveform.png".format(label))
        )
        plt.close()

    def waveform_td_plot(self, label):
        """Generate a time domain waveform plot for a given result file

        Parameters
        ----------
        label: str
            the label corresponding to the results file
        """
        if self.approximant[label] == {}:
            return
        self._waveform_td_plot(
            self.savedir, self.detectors[label], self.maxL_samples[label], label
        )

    @staticmethod
    def _waveform_td_plot(savedir, detectors, maxL_samples, label):
        """Generate a time domain waveform plot for a given detector network
        and set of samples

        Parameters
        ----------
        savedir: str
            the directory you wish to save the plot in
        detectors: list
            list of detectors used in your analysis
        maxL_samples: dict
            dictionary of maximum likelihood values
        label: str
            the label corresponding to the results file
        """
        if detectors is None:
            detectors = ["H1", "L1"]
        else:
            detectors = detectors.split("_")

        fig = gw._time_domain_waveform(detectors, maxL_samples)
        plt.savefig(
            os.path.join(
                savedir, "{}_waveform_time_domain.png".format(label)
            )
        )
        plt.close()

    def strain_plot(self, label):
        """Generate a plot showing the comparison between the data and the
        maxL waveform gfor a given result file

        Parameters
        ----------
        label: str
            the label corresponding to the results file
        """
        from pesummary.utils.utils import RedirectLogger

        logger.info("Launching subprocess to generate strain plot")
        process = mp.Process(
            target=self._strain_plot,
            args=[self.savedir, self.gwdata, self.maxL_samples[label], label]
        )
        process.start()

    @staticmethod
    def _strain_plot(savedir, gwdata, maxL_samples, label):
        """Generate a strain plot for a given set of samples

        Parameters
        ----------
        savedir: str
            the directory to save the plot
        gwdata: dict
            dictionary of strain data for each detector
        maxL_samples: dict
            dictionary of maximum likelihood values
        label: str
            the label corresponding to the results file
        """
        fig = gw._strain_plot(gwdata, maxL_samples)
        plt.savefig(
            os.path.join(savedir, "{}_strain.png".format(label))
        )
        plt.close()

    def skymap_comparison_plot(self, label):
        """Generate a plot to compare skymaps for all result files

        Parameters
        ----------
        label: str
            the label for the results file that you wish to plot
        """
        self._skymap_comparison_plot(
            self.savedir, self.same_samples["ra"], self.same_samples["dec"],
            self.labels, self.colors
        )

    @staticmethod
    def _skymap_comparison_plot(savedir, ra, dec, labels, colors):
        """Generate a plot to compare skymaps for a given set of samples

        Parameters
        ----------
        savedir: str
            the directory you wish to save the plot in
        ra: dict
            dictionary of right ascension samples for each result file
        dec: dict
            dictionary of declination samples for each result file
        labels: list
            list of labels to distinguish each result file
        colors: list
            list of colors to be used to distinguish different result files
        """
        ra_list = [ra[key] for key in labels]
        dec_list = [dec[key] for key in labels]
        fig = gw._sky_map_comparison_plot(ra_list, dec_list, labels, colors)
        plt.savefig(os.path.join(savedir, "combined_skymap.png"))
        plt.close()

    def waveform_comparison_fd_plot(self, label):
        """Generate a plot to compare the frequency domain waveform

        Parameters
        ----------
        label: str
            the label for the results file that you wish to plot
        """
        if any(self.approximant[i] == {} for i in self.labels):
            return

        self._waveform_comparison_fd_plot(
            self.savedir, self.maxL_samples, self.labels, self.colors
        )

    @staticmethod
    def _waveform_comparison_fd_plot(savedir, maxL_samples, labels, colors):
        """Generate a plot to compare the frequency domain waveforms

        Parameters
        ----------
        savedir: str
            the directory you wish to save the plot in
        maxL_samples: dict
            dictionary of maximum likelihood samples for each result file
        labels: list
            list of labels to distinguish each result file
        colors: list
            list of colors to be used to distinguish different result files
        """
        samples = [maxL_samples[i] for i in labels]
        fig = gw._waveform_comparison_plot(samples, colors, labels)
        plt.savefig(os.path.join(savedir, "compare_waveforms.png"))
        plt.close()

    def waveform_comparison_td_plot(self, label):
        """Generate a plot to compare the time domain waveform

        Parameters
        ----------
        label: str
            the label for the results file that you wish to plot
        """
        if any(self.approximant[i] == {} for i in self.labels):
            return

        self._waveform_comparison_fd_plot(
            self.savedir, self.maxL_samples, self.labels, self.colors
        )

    @staticmethod
    def _waveform_comparison_td_plot(savedir, maxL_samples, labels, colors):
        """Generate a plot to compare the time domain waveforms

        Parameters
        ----------
        savedir: str
            the directory you wish to save the plot in
        maxL_samples: dict
            dictionary of maximum likelihood samples for each result file
        labels: list
            list of labels to distinguish each result file
        colors: list
            list of colors to be used to distinguish different result files
        """
        samples = [maxL_samples[i] for i in labels]
        fig = gw._time_domainwaveform_comparison_plot(samples, colors, labels)
        plt.savefig(os.path.join(savedir, "compare_time_domain_waveforms.png"))
        plt.close()

    def twod_comparison_contour_plot(self, label):
        """Generate 2d comparison contour plots

        Parameters
        ----------
        label: str
            the label for the results file that you wish to plot
        """
        error_message = (
            "Failed to generate a 2d contour plot for %s because {}"
        )
        twod_plots = [
            ["mass_ratio", "chi_eff"], ["mass_1", "mass_2"],
            ["luminosity_distance", "chirp_mass_source"],
            ["mass_1_source", "mass_2_source"],
            ["theta_jn", "luminosity_distance"],
            ["network_optimal_snr", "chirp_mass_source"]
        ]
        for plot in twod_plots:
            if not all(
                all(
                    i in self.samples[j].keys() for i in plot
                ) for j in self.labels
            ):
                logger.warn(
                    "Failed to generate 2d contour plots for {} because {} are not "
                    "common in all result files".format(
                        " and ".join(plot), " and ".join(plot)
                    )
                )
                continue
            samples = [[self.samples[i][j] for j in plot] for i in self.labels]
            arguments = [
                self.savedir, plot, samples, self.labels, latex_labels
            ]
            self._try_to_make_a_plot(
                arguments, self._twod_comparison_contour_plot,
                error_message % (" and ".join(plot))
            )

    @staticmethod
    def _twod_comparison_contour_plot(
        savedir, plot_parameters, samples, labels, latex_labels
    ):
        """Generate a 2d comparison contour plot for a given set of samples

        Parameters
        ----------
        savedir: str
            the directory you wish to save the plot in
        plot_parameters: list
            list of parameters to use for the 2d contour plot
        samples: list
            list of samples for each parameter
        labels: list
            list of labels used to distinguish each result file
        latex_labels: dict
            dictionary containing the latex labels for each parameter
        """
        fig = publication.twod_contour_plots(
            plot_parameters, samples, labels, latex_labels
        )
        plt.savefig(
            os.path.join(
                savedir, "publication", "2d_contour_plot_{}.png".format(
                    "_and_".join(plot_parameters)
                )
            )
        )
        plt.close()

    def violin_plot(self, label):
        """Generate violin plot to compare certain parameters in all result
        files

        Parameters
        ----------
        label: str
            the label for the results file that you wish to plot
        """
        error_message = (
            "Failed to generate a violin plot for %s because {}"
        )
        violin_plots = ["mass_ratio", "chi_eff", "chi_p", "luminosity_distance"]

        for plot in violin_plots:
            if not all(plot in self.samples[j].keys() for j in self.labels):
                logger.warn(
                    "Failed to generate violin plots for {} because {} is not "
                    "common in all result files".format(plot, plot)
                )
            samples = [self.samples[i][plot] for i in self.labels]
            arguments = [
                self.savedir, plot, samples, self.labels, latex_labels[plot]
            ]
            self._try_to_make_a_plot(
                arguments, self._violin_plot, error_message % (plot)
            )

    @staticmethod
    def _violin_plot(savedir, plot_parameter, samples, labels, latex_label):
        """Generate a violin plot for a given set of samples

        Parameters
        ----------
        savedir: str
            the directory you wish to save the plot in
        plot_parameter: str
            name of the parameter you wish to generate a violin plot for
        samples: list
            list of samples for each parameter
        labels: list
            list of labels used to distinguish each result file
        latex_label: str
             latex_label correspondig to parameter
        """
        fig = publication.violin_plots(
            plot_parameter, samples, labels, latex_labels
        )
        plt.savefig(
            os.path.join(
                savedir, "publication", "violin_plot_{}.png".format(
                    plot_parameter
                )
            )
        )
        plt.close()

    def spin_dist_plot(self, label):
        """Generate a spin disk plot to compare spins in all result
        files

        Parameters
        ----------
        label: str
            the label for the results file that you wish to plot
        """
        error_message = (
            "Failed to generate a spin disk plot for %s because {}"
        )
        parameters = ["a_1", "a_2", "tilt_1", "tilt_2"]
        for num, label in enumerate(self.labels):
            if not all(i in self.samples[label].keys() for i in parameters):
                logger.warn(
                    "Failed to generate spin disk plots because {} are not "
                    "common in all result files".format(
                        " and ".join(parameters)
                    )
                )
                continue
            samples = [self.samples[label][i] for i in parameters]
            arguments = [
                self.savedir, parameters, samples, label, self.colors[num]
            ]

            self._try_to_make_a_plot(
                arguments, self._spin_dist_plot, error_message % (label)
            )

    @staticmethod
    def _spin_dist_plot(savedir, parameters, samples, label, color):
        """Generate a spin disk plot for a given set of samples

        Parameters
        ----------
        """
        fig = publication.spin_distribution_plots(
            parameters, samples, label, color
        )
        plt.savefig(
            os.path.join(
                savedir, "publication", "spin_disk_plot_{}.png".format(
                    label
                )
            )
        )
        plt.close()

    def pepredicates_plot(self, label):
        """Generate plots with the PEPredicates package

        Parameters
        ----------
        label: str
            the label for the results file that you wish to plot
        """
        self._pepredicates_plot(
            self.savedir, self.samples[label], label,
            self.pepredicates_probs[label]["default"], population_prior=False
        )
        self._pepredicates_plot(
            self.savedir, self.samples[label], label,
            self.pepredicates_probs[label]["population"], population_prior=True
        )

    @staticmethod
    def _pepredicates_plot(
        savedir, samples, label, probabilities, population_prior=False
    ):
        """Generate a plot with the PEPredicates package for a given set of
        samples

        Parameters
        ----------
        savedir: str
            the directory you wish to save the plot in
        samples: dict
            dictionary of samples for each parameter
        label: str
            the label corresponding to the result file
        probabilities: dict
            dictionary of classification probabilities
        population_prior: Bool, optional
            if True, the samples will be reweighted according to a population
            prior
        """
        from pesummary.gw.pepredicates import PEPredicates

        parameters = list(samples.keys())
        samples = [
            [samples[parameter][j] for parameter in parameters] for j in
            range(len(samples[parameters[0]]))
        ]
        fig = PEPredicates.plot(
            samples, parameters, population_prior=population_prior
        )
        if not population_prior:
            plt.savefig(
                os.path.join(
                    savedir, "{}_default_pepredicates.png".format(
                        label
                    )
                )
            )
        else:
            plt.savefig(
                os.path.join(
                    savedir, "{}_population_pepredicates.png".format(
                        label
                    )
                )
            )
        fig = gw._classification_plot(probabilities)
        if not population_prior:
            plt.savefig(
                os.path.join(
                    savedir, "{}_default_pepredicates_bar.png".format(
                        label
                    )
                )
            )
        else:
            plt.savefig(
                os.path.join(
                    savedir, "{}_population_pepredicates_bar.png".format(
                        label
                    )
                )
            )
        plt.close()

    def psd_plot(self, label):
        """Generate a psd plot for a given result file

        Parameters
        ----------
        label: str
            the label corresponding to the result file
        """
        error_message = (
            "Failed to generate a PSD plot for %s because {}"
        )

        fmin = None

        for num, label in enumerate(self.labels):
            if list(self.psd[label].keys()) == [None]:
                return
            if list(self.psd[label].keys()) == []:
                return
            if "f_low" in list(self.file_kwargs[label]["sampler"].keys()):
                fmin = self.file_kwargs[label]["sampler"]["f_low"]
            labels = list(self.psd[label].keys())
            frequencies = [np.array(self.psd[label][i]).T[0] for i in labels]
            strains = [np.array(self.psd[label][i]).T[1] for i in labels]
            arguments = [
                self.savedir, frequencies, strains, fmin, labels, label
            ]

            self._try_to_make_a_plot(
                arguments, self._psd_plot, error_message % (label)
            )

    @staticmethod
    def _psd_plot(savedir, frequencies, strains, fmin, psd_labels, label):
        """Generate a psd plot for a given set of samples

        Parameters
        ----------
        savedir: str
            the directory you wish to save the plot in
        frequencies: list
            list of psd frequencies for each IFO
        strains: list
            list of psd strains for each IFO
        fmin: float
            frequency to start the psd plotting
        psd_labels: list
            list of IFOs used
        label: str
            the label used to distinguish the result file
        """
        fig = gw._psd_plot(
            frequencies, strains, labels=psd_labels, fmin=fmin
        )
        plt.savefig(
            os.path.join(savedir, "{}_psd_plot.png".format(label))
        )
        plt.close()

    def calibration_plot(self, label):
        """Generate a calibration plot for a given result file

        Parameters
        ----------
        label: str
            the label corresponding to the result file
        """
        import numpy as np

        error_message = (
            "Failed to generate calibration plot for %s because {}"
        )
        frequencies = np.arange(20., 1024., 1. / 4)

        for num, label in enumerate(self.labels):
            if list(self.calibration[label].keys()) == [None]:
                return
            if list(self.calibration[label].keys()) == []:
                return

            ifos = list(self.calibration[label].keys())
            calibration_data = [
                self.calibration[label][i] for i in ifos
            ]
            if "calibration" in self.priors.keys():
                prior = [self.priors["calibration"][label][i] for i in ifos]
            else:
                prior = None
            arguments = [
                self.savedir, frequencies, calibration_data, ifos, prior,
                label
            ]
            self._try_to_make_a_plot(
                arguments, self._calibration_plot, error_message % (label)
            )

    @staticmethod
    def _calibration_plot(
        savedir, frequencies, calibration_data, calibration_labels, prior, label
    ):
        """Generate a calibration plot for a given set of samples

        Parameters
        ----------
        savedir: str
            the directory you wish to save the plot in
        frequencies: list
            list of frequencies used to interpolate the calibration data
        calibration_data: list
            list of calibration data for each IFO
        calibration_labels: list
            list of IFOs used
        prior: list
            list containing the priors used for each IFO
        label: str
            the label used to distinguish the result file
        """
        fig = gw._calibration_envelope_plot(
            frequencies, calibration_data, calibration_labels, prior=prior
        )
        plt.savefig(
            os.path.join(savedir, "{}_calibration_plot.png".format(label))
        )
        plt.close()



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import os

import h5py
import numpy as np
try:
    from glue.ligolw import ligolw
    from glue.ligolw import lsctables
    from glue.ligolw import utils as ligolw_utils
    GLUE = True
except ImportError:
    GLUE = False

from pesummary.gw.file.formats.base_read import GWRead
from pesummary.gw.file import conversions as con
from pesummary.utils.utils import logger


class LALInference(GWRead):
    """PESummary wrapper of `lalinference`
    (https://git.ligo.org/lscsoft/lalsuite/lalinference).

    Attributes
    ----------
    path_to_results_file: str
        path to the results file you wish to load in with `LALInference`
    """
    def __init__(self, path_to_results_file, injection_file=None):
        super(LALInference, self).__init__(path_to_results_file)
        self.load(self._grab_data_from_lalinference_file)

    @classmethod
    def load_file(cls, path, injection_file=None):
        if not os.path.isfile(path):
            raise IOError("%s does not exist" % (path))
        if injection_file and not os.path.isfile(injection_file):
            raise IOError("%s does not exist" % (path))
        return cls(path, injection_file=injection_file)

    @staticmethod
    def guess_path_to_sampler(path):
        """Guess the path to the sampler group in a LALInference results file

        Parameters
        ----------
        path: str
            path to the LALInference results file
        """
        def _find_name(name):
            c1 = "lalinference_nest" in name or "lalinference_mcmc" in name
            c2 = "lalinference_nest/" not in name and "lalinference_mcmc/" not in name
            if c1 and c2:
                return name

        f = h5py.File(path, 'r')
        _path = f.visit(_find_name)
        f.close()
        return _path

    @staticmethod
    def _parameters_in_lalinference_file(path):
        """Return the parameter names stored in the LALInference results file

        Parameters
        ----------
        """
        f = h5py.File(path, 'r')
        path_to_samples = GWRead.guess_path_to_samples(path)
        parameters = list(f[path_to_samples].dtype.names)
        f.close()
        return parameters

    @staticmethod
    def _samples_in_lalinference_file(path):
        """
        """
        f = h5py.File(path, 'r')
        path_to_samples = GWRead.guess_path_to_samples(path)
        samples = [list(i) for i in f[path_to_samples]]
        return samples

    @staticmethod
    def _check_for_calibration_data_in_lalinference_file(path):
        """
        """
        f = h5py.File(path, 'r')
        path_to_samples = GWRead.guess_path_to_samples(path)
        lalinference_names = list(f[path_to_samples].dtype.names)
        if any("_spcal_amp" in i for i in lalinference_names):
            return True
        return False

    @property
    def calibration_data_in_results_file(self):
        """
        """
        check = LALInference._check_for_calibration_data_in_lalinference_file
        grab = LALInference._grab_calibration_data_from_lalinference_file
        if self.check_for_calibration_data(check, self.path_to_results_file):
            return self.grab_calibration_data(grab, self.path_to_results_file)
        return None

    @staticmethod
    def grab_extra_kwargs(path):
        """Grab any additional information stored in the lalinference file
        """
        kwargs = {"sampler": {}, "meta_data": {}}
        path_to_samples = GWRead.guess_path_to_samples(path)
        path_to_sampler = LALInference.guess_path_to_sampler(path)
        f = h5py.File(path, 'r')
        attributes = dict(f[path_to_samples].attrs.items())
        if "flow" in list(attributes.keys()):
            kwargs["sampler"]["f_low"] = attributes["flow"]
        elif "f_low" in list(attributes.keys()):
            kwargs["sampler"]["f_low"] = attributes["f_low"]
        if "fref" in list(attributes.keys()):
            kwargs["sampler"]["f_ref"] = attributes["fref"]
        elif "f_ref" in list(attributes.keys()):
            kwargs["sampler"]["f_ref"] = attributes["f_ref"]
        if "LAL_APPROXIMANT" in list(attributes.keys()):
            try:
                from lalsimulation import GetStringFromApproximant

                kwargs["meta_data"]["approximant"] = \
                    GetStringFromApproximant(int(attributes["LAL_APPROXIMANT"]))
            except Exception:
                kwargs["meta_data"]["approximant"] = \
                    int(attributes["LAL_APPROXIMANT"])
        if "number_of_live_points" in list(attributes.keys()):
            kwargs["meta_data"]["number_of_live_points"] = attributes["number_of_live_points"]
        if "segmentLength" in list(attributes.keys()):
            kwargs["meta_data"]["seglen"] = attributes["segmentLength"]
        if "sampleRate" in list(attributes.keys()):
            kwargs["meta_data"]["samplerate"] = attributes["sampleRate"]

        attributes = dict(f[path_to_sampler].attrs.items())
        if "log_bayes_factor" in list(attributes.keys()):
            kwargs["sampler"]["log_bayes_factor"] = np.round(
                attributes["log_bayes_factor"], 2)
        elif "bayes_factor" in list(attributes.keys()):
            kwargs["sampler"]["log_bayes_factor"] = np.round(
                np.log(attributes["log_bayes_factor"]), 2)
        if "log_evidence" in list(attributes.keys()):
            kwargs["sampler"]["log_evidence"] = np.round(
                attributes["log_evidence"], 2)
        elif "evidence" in list(attributes.keys()):
            kwargs["sampler"]["log_evidence"] = np.round(
                np.log(attributes["evidence"]), 2)
        if "log_prior_volume" in list(attributes.keys()):
            kwargs["sampler"]["log_prior_volume"] = np.round(
                attributes["log_prior_volume"], 2)
        elif "prior_volume" in list(attributes.keys()):
            kwargs["sampler"]["log_prior_volume"] = np.round(
                np.log(attributes["prior_volume"]), 2)
        if "number_of_live_points" in list(attributes.keys()):
            kwargs["meta_data"]["number_of_live_points"] = attributes["number_of_live_points"]
        if "segmentLength" in list(attributes.keys()):
            kwargs["meta_data"]["seglen"] = attributes["segmentLength"]
        if "sampleRate" in list(attributes.keys()):
            kwargs["meta_data"]["samplerate"] = attributes["sampleRate"]
        f.close()
        return kwargs

    @staticmethod
    def _grab_calibration_data_from_lalinference_file(path):
        """
        """
        f = h5py.File(path, 'r')
        path_to_samples = GWRead.guess_path_to_samples(path)
        attributes = f[path_to_samples].attrs.items()
        lalinference_names = list(f[path_to_samples].dtype.names)
        samples = [list(i) for i in f[path_to_samples]]
        keys_amp = np.sort([
            param for param in lalinference_names if "_spcal_amp" in param])
        keys_phase = np.sort([
            param for param in lalinference_names if "_spcal_phase" in
            param])
        log_frequencies = {
            key.split("_")[0]: [] for key, value in attributes if
            "_spcal_logfreq" in key or "_spcal_freq" in key}
        for key, value in attributes:
            if "_spcal_logfreq" in key:
                log_frequencies[key.split("_")[0]].append(float(value))
            if "_spcal_freq" in key:
                log_frequencies[key.split("_")[0]].append(np.log(float(value)))

        amp_params = {ifo: [] for ifo in log_frequencies.keys()}
        phase_params = {ifo: [] for ifo in log_frequencies.keys()}
        for key in keys_amp:
            ifo = key.split("_")[0]
            ind = lalinference_names.index(key)
            amp_params[ifo].append([float(i[ind]) for i in samples])
        for key in keys_phase:
            ifo = key.split("_")[0]
            ind = lalinference_names.index(key)
            phase_params[ifo].append([float(i[ind]) for i in samples])
        f.close()
        return log_frequencies, amp_params, phase_params

    @staticmethod
    def _grab_data_from_lalinference_file(path):
        """
        """
        f = h5py.File(path, 'r')
        path_to_samples = GWRead.guess_path_to_samples(path)
        lalinference_names = list(f[path_to_samples].dtype.names)
        samples = [list(i) for i in f[path_to_samples]]

        if "logdistance" in lalinference_names:
            lalinference_names.append("luminosity_distance")
            for num, i in enumerate(samples):
                samples[num].append(
                    np.exp(i[lalinference_names.index("logdistance")]))
        if "costheta_jn" in lalinference_names:
            lalinference_names.append("theta_jn")
            for num, i in enumerate(samples):
                samples[num].append(
                    np.arccos(i[lalinference_names.index("costheta_jn")]))
        extra_kwargs = LALInference.grab_extra_kwargs(path)
        extra_kwargs["sampler"]["nsamples"] = len(samples)
        try:
            version = f[path_to_samples].attrs["VERSION"].decode("utf-8")
        except Exception as e:
            version = None
        return {
            "parameters": lalinference_names,
            "samples": samples,
            "injection": None,
            "version": version,
            "kwargs": extra_kwargs
        }

    def add_injection_parameters_from_file(self, injection_file):
        """
        """
        self.injection_parameters = self._add_injection_parameters_from_file(
            injection_file, self._grab_injection_parameters_from_file)

    def add_fixed_parameters_from_config_file(self, config_file):
        """Search the conifiguration file and add fixed parameters to the
        list of parameters and samples

        Parameters
        ----------
        config_file: str
            path to the configuration file
        """
        self._add_fixed_parameters_from_config_file(
            config_file, self._add_fixed_parameters)

    def add_marginalized_parameters_from_config_file(self, config_file):
        """Search the configuration file and add the marginalized parameters
        to the list of parameters and samples

        Parameters
        ----------
        config_file: str
            path to the configuration file
        """
        self._add_marginalized_parameters_from_config_file(
            config_file, self._add_marginalized_parameters)

    @staticmethod
    def _add_fixed_parameters(parameters, samples, config_file):
        """Open a LALInference configuration file and add the fixed parameters
        to the list of parameters and samples

        Parameters
        ----------
        parameters: list
            list of existing parameters
        samples: list
            list of existing samples
        config_file: str
            path to the configuration file
        """
        import configparser
        from pesummary.gw.file.standard_names import standard_names

        config = configparser.ConfigParser()
        try:
            config.read(config_file)
            fixed_data = None
            if "engine" in config.sections():
                fixed_data = {
                    key.split("fix-")[1]: item for key, item in
                    config.items("engine") if "fix" in key}
            for i in fixed_data.keys():
                fixed_parameter = i
                fixed_value = fixed_data[i]
                try:
                    param = standard_names[fixed_parameter]
                    if param in parameters:
                        pass
                    else:
                        parameters.append(param)
                        for num in range(len(samples)):
                            samples[num].append(float(fixed_value))
                except Exception:
                    if fixed_parameter == "logdistance":
                        if "luminosity_distance" not in parameters:
                            parameters.append(standard_names["distance"])
                            for num in range(len(samples)):
                                samples[num].append(float(fixed_value))
                    if fixed_parameter == "costheta_jn":
                        if "theta_jn" not in parameters:
                            parameters.append(standard_names["theta_jn"])
                            for num in range(len(samples)):
                                samples[num].append(float(fixed_value))
            return parameters, samples
        except Exception:
            return parameters, samples

    @staticmethod
    def _add_marginalized_parameters(parameters, samples, config_file):
        """Open a LALInference configuration file and add the marginalized
        parameters to the list of parameters and samples

        Parameters
        ----------
        parameters: list
            list of existing parameters
        samples: list
            list of existing samples
        config_file: str
            path to the configuration file
        """
        import configparser
        from pesummary.gw.file.standard_names import standard_names

        config = configparser.ConfigParser()
        try:
            config.read(config_file)
            fixed_data = None
            if "engine" in config.sections():
                marg_par = {
                    key.split("marg")[1]: item for key, item in
                    config.items("engine") if "marg" in key}
            for i in marg_par.keys():
                if "time" in i and "geocent_time" not in parameters:
                    if "marginalized_geocent_time" in parameters:
                        ind = parameters.index("marginalized_geocent_time")
                        parameters.remove(parameters[ind])
                        parameters.append("geocent_time")
                        for num, j in enumerate(samples):
                            samples[num].append(float(j[ind]))
                            del j[ind]
                    else:
                        logger.warn("You have marginalized over time and "
                                    "there are no time samples. Manually "
                                    "setting time to 100000s")
                        parameters.append("geocent_time")
                        for num, j in enumerate(samples):
                            samples[num].append(float(100000))
                if "phi" in i and "phase" not in parameters:
                    if "marginalized_phase" in parameters:
                        ind = parameters.index("marginalized_phase")
                        parameters.remove(parameters[ind])
                        parameters.append("phase")
                        for num, j in enumerate(samples):
                            samples[num].append(float(j[ind]))
                            del j[ind]
                    else:
                        logger.warn("You have marginalized over phase and "
                                    "there are no phase samples. Manually "
                                    "setting the phase to be 0")
                        parameters.append("phase")
                        for num, j in enumerate(samples):
                            samples[num].append(float(0))
                if "dist" in i and "luminosity_distance" not in parameters:
                    if "marginalized_distance" in parameters:
                        ind = parameters.index("marginalized_distance")
                        parameters.remove(parameters[ind])
                        parameters.append("luminosity_distance")
                        for num, j in enumerate(samples):
                            samples[num].append(float(j[ind]))
                            del j[ind]
                    else:
                        logger.warn("You have marginalized over distance and "
                                    "there are no distance samples. Manually "
                                    "setting distance to 100Mpc")
                        parameters.append("luminosity_distance")
                        for num, j in enumerate(samples):
                            samples[num].append(float(100.0))
            return parameters, samples
        except Exception:
            return parameters, samples



# Copyright (C) 2019 Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import os
from glob import glob
from scipy import stats
import numpy as np

import pesummary
from pesummary.utils.utils import logger
from pesummary.core.webpage import webpage


class _WebpageGeneration(object):
    """Super class to handle the webpage generation for a given set of result
    files

    Parameters
    ----------
    savedir: str
        the directory to store the plots
    webdir: str
        the web directory of the run
    samples: dict
        dictionary of posterior samples stored in the result files
    labels: list
        list of labels used to distinguish the result files
    publication: Bool
        Bool to determine the generation of a publication webpage
    user: str
        the user that submitted the job
    config: list
        list of configuration files used for each result file
    same_parameters: list
        list of paramerers that are common in all result files
    base_url: str
        the url corresponding to the web directory
    file_versions: dict
        dictionary of file versions for each result file
    hdf5: Bool
        Bool to determine if the metafile has been saved as hdf5 or not
    colors: list
        colors that you wish to use to distinguish different result files
    custom_plotting:
    existing_labels: list
        list of labels stored in an existing metafile
    existing_config: list
        list of configuration files stored in the existing metafile
    existing_file_version: dict
        dictionary of file versions stored in the existing metafile
    existing_injection_data: dict
        dictionary of injection data stored in an existing metafile
    existing_samples: dict
        dictionary of posterior samples stored in an existing metafile
    existing_metafile: str
        path to the existing metafile
    existing_file_kwargs: dict
        dictionary of file kwargs stored in an existing metafile
    add_to_existing: Bool
        Bool to determine if you wish to add to an existing webpage
    notes: str
        notes that you wish to put on the webpages
    """
    def __init__(
        self, webdir=None, samples=None, labels=None, publication=None,
        user=None, config=None, same_parameters=None, base_url=None,
        file_versions=None, hdf5=None, colors=None, custom_plotting=None,
        existing_labels=None, existing_config=None, existing_file_version=None,
        existing_injection_data=None, existing_samples=None,
        existing_metafile=None, existing_file_kwargs=None,
        add_to_existing=False, notes=None
    ):
        self.webdir = webdir
        self.samples = samples
        self.labels = labels
        self.publication = publication
        self.user = user
        self.config = config
        self.same_parameters = same_parameters
        self.base_url = base_url
        self.file_versions = file_versions
        self.hdf5 = hdf5
        self.colors = colors
        self.custom_plotting = custom_plotting
        self.existing_labels = existing_labels
        self.existing_config = existing_config
        self.existing_file_version = existing_file_version
        self.existing_samples = existing_samples
        self.existing_metafile = existing_metafile
        self.existing_file_kwargs = existing_file_kwargs
        self.add_to_existing = add_to_existing
        self.notes = notes
        self.categories = self.default_categories()
        self.popular_options = self.default_popular_options()
        self.navbar = {
            "home": self.make_navbar_for_homepage(),
            "result_page": self.make_navbar_for_result_page(),
            "comparison": self.make_navbar_for_comparison_page()
        }
        self.image_path = {
            "home": os.path.join(".", "plots", ""),
            "other": os.path.join("..", "plots", "")
        }
        self.results_path = {
            "home": "./samples/", "other": "../samples/"
        }
        if len(self.samples) > 1:
            try:
                self.comparison_stats = self.generate_comparison_statistics()
            except Exception as e:
                self.comparison_stats = None
                logger.info(
                    "Failed to generate comparison statistics because {}. As a "
                    "result they will not be added to the webpages".format(e)
                )

    def generate_comparison_statistics(self):
        """Generate comparison statistics for all parameters that are common to
        all result files
        """
        data = {
            i: self._generate_comparison_statistics(
                [self.samples[j][i] for j in self.labels]
            ) for i in self.same_parameters
        }
        return data

    def _generate_comparison_statistics(self, samples):
        """Generate comparison statistics for a set of samples

        Parameters
        ----------
        samples: list
            list of samples for each result file
        """
        from scipy.stats import gaussian_kde

        rows = range(len(samples))
        columns = range(len(samples))
        kernel = [gaussian_kde(i) for i in samples]
        x = np.linspace(
            np.min([np.min(i) for i in samples]),
            np.max([np.max(i) for i in samples]),
            100
        )
        ks = [
            [
                self.kolmogorov_smirnov_test(samples[i], samples[j]) for i in
                rows
            ] for j in columns
        ]
        js = [
            [
                self.jension_shannon_divergence(kernel[i](x), kernel[j](x)) for
                i in rows
            ] for j in columns
        ]
        return [ks, js]

    @staticmethod
    def get_executable(executable):
        """Return the path to an executable

        Parameters
        ----------
        executable: str
            the name of the executable you wish to find
        """
        from subprocess import check_output

        path = check_output(["which", executable]).decode("utf-8").strip()
        return path

    @staticmethod
    def kolmogorov_smirnov_test(a, b):
        """Return the KS p value between two PDFs

        Parameters
        ----------
        a: list
            List containing the first PDF that you would like to compare
        b: list
            List containing the second PDF that you would like to compare
        """
        return stats.ks_2samp(a, b)[1]

    @staticmethod
    def jension_shannon_divergence(a, b):
        """Return the JS divergence test between two PDFs

        Parameters
        ----------
        a: list
            List containing the first PDF that you would like to compare
        b: list
            List containing the second PDF that you would like to compare
        """
        a = np.asarray(a)
        b = np.asarray(b)
        a /= a.sum()
        b /= b.sum()
        m = 1. / 2 * (a + b)
        kl_forward = stats.entropy(a, qk=m)
        kl_backward = stats.entropy(b, qk=m)
        return kl_forward / 2. + kl_backward / 2.

    def _result_page_links(self):
        """Return the navbar structure for the Result Page tab.
        """
        return [{i: i} for i in self.labels]

    def make_navbar_for_homepage(self):
        """Make a navbar for the homepage
        """
        links = [
            "home", ["Result Pages", self._result_page_links()], "Logging",
            "Version"
        ]
        if len(self.samples) > 1:
            links[1][1] += ["Comparison"]
        if self.publication:
            links.insert(2, "Publication")
        if self.notes is not None:
            links.append("Notes")
        return links

    def make_navbar_for_result_page(self):
        """Make a navbar for the result page homepage
        """
        links = {
            i: ["1d Histograms", [{"Multiple": i}]] for i in self.labels
        }
        for num, label in enumerate(self.labels):
            for j in self.categorize_parameters(self.samples[label].keys()):
                j = [j[0], [{k: label} for k in j[1]]]
                links[label].append(j)

        final_links = {
            i: [
                "home", ["Result Pages", self._result_page_links()],
                {"Corner": i}, {"Config": i}, links[i]
            ] for i in self.labels
        }
        if len(self.samples) > 1:
            for label in self.labels:
                final_links[label][1][1] += ["Comparison"]
        return final_links

    def make_navbar_for_comparison_page(self):
        """Make a navbar for the comparison homepage
        """
        if self.same_parameters is not None:
            links = ["1d Histograms", ["Multiple"]]
            for i in self.categorize_parameters(self.same_parameters):
                links.append(i)
            final_links = [
                "home", ["Result Pages", self._result_page_links()], links
            ]
            final_links[1][1] += ["Comparison"]
            return final_links
        return None

    def categorize_parameters(self, parameters):
        """Categorize the parameters into common headings

        Parameters
        ----------
        parameters: list
            list of parameters that you would like to sort
        """
        params = []
        for heading, category in self.categories.items():
            if any(
                any(i[0] in j for j in category["accept"]) for i in parameters
            ):
                cond = self._condition(category["accept"], category["reject"])
                params.append(
                    [heading, self._partition(cond, parameters)]
                )
        return params

    def _condition(self, true, false):
        """Setup a condition

        Parameters
        ----------
        true: list
            list of strings that you would like to include
        false: list
            list of strings that you would like to neglect
        """
        if len(true) != 0 and len(false) == 0:
            condition = lambda j: True if any(i in j[0] for i in true) else \
                False
        elif len(true) == 0 and len(false) != 0:
            condition = lambda j: True if any(i not in j[0] for i in false) \
                else False
        elif len(true) and len(false) != 0:
            condition = lambda j: True if any(i in j[0] for i in true) and \
                any(i not in j[0] for i in false) else False
        return condition

    def _partition(self, condition, array):
        """Filter the list according to a condition

        Parameters
        ----------
        condition: func
            lambda function containing the condition that you want to use to
            filter the array
        array: list
            List of parameters that you would like to filter
        """
        return sorted(list(filter(condition, array)))

    def generate_webpages(self):
        """Generate all webpages for all result files passed
        """
        if self.add_to_existing:
            self.add_existing_data()
        self.make_home_pages()
        self.make_1d_histogram_pages()
        self.make_corner_pages()
        self.make_config_pages()
        if len(self.labels) > 1:
            self.make_comparison_pages()
        self.make_error_page()
        self.make_version_page()
        self.make_logging_page()
        if self.notes is not None:
            self.make_notes_page()
        self.generate_specific_javascript()

    def create_blank_html_pages(self, pages, stylesheets=[]):
        """Create blank html pages

        Parameters
        ----------
        pages: list
            list of pages that you wish to create
        """
        webpage.make_html(
            web_dir=self.webdir, pages=pages, stylesheets=stylesheets
        )

    def setup_page(
        self, html_page, links, label=None, title=None, approximant=None,
        background_colour=None, histogram_download=False
    ):
        """Set up each webpage with a header and navigation bar.

        Parameters
        ----------
        html_page: str
            String containing the html page that you would like to set up
        links: list
            List containing the navbar structure that you would like to include
        label: str, optional
            The label that prepends your webpage name
        title: str, optional
            String that you would like to include in your header
        approximant: str, optional
            The approximant that you would like associated with your html_page
        background_colour: str, optional
            String containing the background colour of your header
        histogram_download: bool, optional
            If true, a download link for the each histogram is displayed in
            the navbar
        """
        html_file = webpage.open_html(
            web_dir=self.webdir, base_url=self.base_url, html_page=html_page,
            label=label
        )
        html_file.make_header(approximant=approximant)
        if html_page == "home" or html_page == "home.html":
            html_file.make_navbar(
                links=links, samples_path=self.results_path["home"],
                background_color=background_colour,
                hdf5=self.hdf5
            )
        elif histogram_download:
            html_file.make_navbar(
                links=links, samples_path=self.results_path["other"],
                histogram_download=os.path.join(
                    "..", "samples", "dat", label, "{}_{}_samples.dat".format(
                        label, html_page
                    )
                ), background_color=background_colour, hdf5=self.hdf5
            )
        else:
            html_file.make_navbar(
                links=links, samples_path=self.results_path["home"],
                background_color=background_colour, hdf5=self.hdf5
            )
        return html_file

    def make_home_pages(self):
        """Wrapper function for _make_home_pages()
        """
        pages = ["{}_{}".format(i, i) for i in self.labels]
        pages.append("home")
        self.create_blank_html_pages(pages)
        self._make_home_pages(pages)

    def _make_home_pages(self, pages):
        """Make the home pages

        Parameters
        ----------
        pages: list
            list of pages that you wish to create
        """
        html_file = self.setup_page("home", self.navbar["home"])
        html_file.make_banner(approximant="Summary", key="Summary")

        for num, i in enumerate(self.labels):
            html_file = self.setup_page(
                i, self.navbar["result_page"][i], i,
                title="{} Summary page".format(i),
                background_colour=self.colors[num], approximant=i
            )
            html_file.make_banner(approximant=i, key=i)
            if self.custom_plotting:
                custom_plots = glob(
                    "{}/plots/{}_custom_plotting_*".format(self.webdir, i)
                )
                path = self.image_path["other"]
                for num, i in enumerate(custom_plots):
                    custom_plots[num] = path + i.split("/")[-1]
                image_contents = [
                    custom_plots[i:4 + i] for i in range(0, len(custom_plots), 4)]
                html_file.make_table_of_images(contents=image_contents)
                images = [y for x in image_contents for y in x]
                html_file.make_modal_carousel(images=images)
            html_file.make_footer(user=self.user, rundir=self.webdir)
            html_file.close()

    def make_1d_histogram_pages(self):
        """Wrapper function for _make_1d_histogram pages
        """
        pages = [
            "{}_{}_{}".format(i, i, j) for i in self.labels for j in
            self.samples[i].keys()
        ]
        pages += ["{}_{}_Multiple".format(i, i) for i in self.labels]
        self.create_blank_html_pages(pages)
        self._make_1d_histogram_pages(pages)

    def _make_1d_histogram_pages(self, pages):
        """Make the 1d histogram pages

        Parameters
        ----------
        pages: list
            list of pages that you wish to create
        """
        for num, i in enumerate(self.labels):
            for j in self.samples[i].keys():
                html_file = self.setup_page(
                    "{}_{}".format(i, j), self.navbar["result_page"][i],
                    i, title="{} Posterior PDF for {}".format(i, j),
                    approximant=i, background_colour=self.colors[num],
                    histogram_download=True
                )
                html_file.make_banner(approximant=i, key=i)
                path = self.image_path["other"]
                contents = [
                    [path + "{}_1d_posterior_{}.png".format(i, j)],
                    [
                        path + "{}_sample_evolution_{}.png".format(i, j),
                        path + "{}_autocorrelation_{}.png".format(i, j)
                    ]
                ]
                html_file.make_table_of_images(
                    contents=contents, rows=1, columns=2, code="changeimage")
                html_file.make_footer(user=self.user, rundir=self.webdir)
                html_file.close()
            html_file = self.setup_page(
                "{}_Multiple".format(i), self.navbar["result_page"][i],
                i, title="{} Posteriors for multiple".format(i),
                approximant=i, background_colour=self.colors[num]
            )
            html_file.make_banner(approximant=i, key=i)
            ordered_parameters = self.categorize_parameters(
                self.samples[i].keys()
            )
            ordered_parameters = [i for j in ordered_parameters for i in j[1]]
            popular_options = self.popular_options
            popular_options.append({"all": ", ".join(ordered_parameters)})
            html_file.make_search_bar(
                sidebar=[i for i in self.samples[i].keys()],
                popular_options=popular_options,
                label=self.labels[num], code="combines"
            )
            html_file.make_footer(user=self.user, rundir=self.webdir)
            html_file.close()

    def make_corner_pages(self):
        """Wrapper function for _make_corner_pages
        """
        pages = ["{}_{}_Corner".format(i, i) for i in self.labels]
        self.create_blank_html_pages(pages)
        self._make_corner_pages(pages)

    def _make_corner_pages(self, pages):
        """Make the corner pages

        Parameters
        ----------
        pages: list
            list of pages that you wish to create
        """
        for num, i in enumerate(self.labels):
            html_file = self.setup_page(
                "{}_Corner".format(i), self.navbar["result_page"][i], i,
                title="{} Corner Plots".format(i), approximant=i,
                background_colour=self.colors[num]
            )
            html_file.make_banner(approximant=i, key="corner")
            html_file.make_search_bar(
                sidebar=self.samples[i].keys(),
                popular_options=self.popular_options, label=i
            )
            html_file.make_footer(user=self.user, rundir=self.webdir)
            html_file.close()

    def make_config_pages(self):
        """Wrapper function for _make_config_pages
        """
        pages = ["{}_{}_Config".format(i, i) for i in self.labels]
        self.create_blank_html_pages(pages, stylesheets=pages)
        self._make_config_pages(pages)

    def _make_config_pages(self, pages):
        """Make the config pages

        Parameters
        ----------
        pages: list
            list of pages that you wish to create
        """
        for num, i in enumerate(self.labels):
            html_file = self.setup_page(
                "{}_Config".format(i), self.navbar["result_page"][i], i,
                title="{} Configuration".format(i), approximant=i,
                background_colour=self.colors[num]
            )
            html_file.make_banner(approximant=i, key="config")
            if self.config and num < len(self.config) and self.config[num]:
                with open(self.config[num], 'r') as f:
                    contents = f.read()
                html_file.make_container()
                styles = html_file.make_code_block(
                    language='ini', contents=contents
                )
                html_file.end_container()
                with open(
                    "{0:s}/css/{1:s}_{2:s}_Config.css".format(
                        self.webdir, i, i
                    ), "w"
                ) as f:
                    f.write(styles)
            else:
                html_file.add_content(
                    "<div class='row justify-content-center'; "
                    "style='font-family: Arial-body; font-size: 14px'>"
                    "<p style='margin-top:2.5em'> No configuration file was "
                    "provided </p></div>"
                )
            html_file.make_footer(user=self.user, rundir=self.webdir)
            html_file.close()

    def make_comparison_pages(self):
        """Wrapper function for _make_comparison_pages
        """
        pages = ["Comparison_{}".format(i) for i in self.same_parameters]
        pages += ["Comparison_Multiple"]
        pages += ["Comparison"]
        self.create_blank_html_pages(pages)
        self._make_comparison_pages(pages)

    def _make_comparison_pages(self, pages):
        """Make pages to compare all result files

        Parameters
        ----------
        pages: list
            list of pages that you wish to create
        """
        html_file = self.setup_page(
            "Comparison", self.navbar["comparison"], approximant="Comparison",
            title="Comparison Summary Page"
        )
        html_file.make_banner(approximant="Comparison", key="Comparison")
        path = self.image_path["other"]
        if self.custom_plotting:
            custom_plots = glob(
                "{}/plots/combined_custom_plotting_*".format(self.webdir)
            )
            for num, i in enumerate(custom_plots):
                custom_plots[num] = path + i.split("/")[-1]
            image_contents = [
                custom_plots[i:4 + 1] for i in range(0, len(custom_plots), 4)
            ]
            html_file.make_table_of_images(contents=image_contents)
            images = [y for x in image_contents for y in x]
            html_file.make_modal_carousel(images=images)
        path = self.image_path["other"]
        if self.comparison_stats is not None:
            rows = range(len(self.labels))
            style = (
                "margin-top:0em; margin-bottom:{}; background-color:#FFFFFF; "
                "box-shadow: 0 0 5px grey;"
            )
            table_contents = {
                i: [
                    [self.labels[j]] + self.comparison_stats[i][0][j] for j in
                    rows
                ] for i in self.same_parameters
            }
            html_file.make_table(
                headings=[" "] + self.labels, contents=table_contents,
                heading_span=1, accordian_header="KS test total",
                style=style.format("1em")
            )
            table_contents = {
                i: [
                    [self.labels[j]] + self.comparison_stats[i][1][j] for j in
                    rows
                ] for i in self.same_parameters
            }
            html_file.make_table(
                headings=[" "] + self.labels, contents=table_contents,
                heading_span=1, accordian_header="JS divergence test total",
                style=style.format("5em")
            )
        html_file.make_footer(user=self.user, rundir=self.webdir)
        for i in self.same_parameters:
            html_file = self.setup_page(
                "Comparison_{}".format(i), self.navbar["comparison"],
                title="Comparison PDF for {}".format(i),
                approximant="Comparison"
            )
            html_file.make_banner(approximant="Comparison", key="Comparison")

            path = self.image_path["other"]
            contents = [
                [path + "combined_1d_posterior_{}.png".format(i)],
                [
                    path + "combined_cdf_{}.png".format(i),
                    path + "combined_boxplot_{}.png".format(i)
                ]
            ]
            html_file.make_table_of_images(
                contents=contents, rows=1, columns=2, code="changeimage"
            )
            if self.comparison_stats is not None:
                table_contents = [
                    [self.labels[j]] + self.comparison_stats[i][0][j] for j in rows
                ]
                html_file.make_table(
                    headings=[" "] + self.labels, contents=table_contents,
                    heading_span=1, accordian_header="KS test",
                    style=style.format("1em")
                )
                table_contents = [
                    [self.labels[j]] + self.comparison_stats[i][1][j] for j in rows
                ]
                html_file.make_table(
                    headings=[" "] + self.labels, contents=table_contents,
                    heading_span=1, accordian_header="JS divergence test",
                    style=style.format("1em")
                )
            html_file.make_footer(user=self.user, rundir=self.webdir)
            html_file.close()
        html_file = self.setup_page(
            "Comparison_Multiple", self.navbar["comparison"],
            approximant="Comparison", title="Comparison Posteriors for multiple"
        )
        html_file.make_search_bar(
            sidebar=self.same_parameters, label="None", code="combines",
            popular_options=[
                {"all": ", ".join(self.same_parameters)}
            ]
        )
        html_file.make_footer(user=self.user, rundir=self.webdir)
        html_file.close()

    def make_error_page(self):
        """Wrapper function for _make_error_page
        """
        pages = ["error"]
        self.create_blank_html_pages(pages)
        self._make_error_page(pages)

    def _make_error_page(self, pages):
        """Make a page that is shown when something goes wrong

        Parameters
        ----------
        pages: list
            list of pages that you wish to create
        """
        webpage.make_html(web_dir=self.webdir, pages=pages)
        html_file = webpage.open_html(
            web_dir=self.webdir, base_url=self.base_url, html_page="error"
        )
        html_file.make_div(
            _class="jumbotron text-center",
            _style="background-color: #D3D3D3; margin-bottom:0"
        )
        html_file.make_div(
            _class="container",
            _style="margin-top:1em; background-color: #D3D3D3; width: 45em"
        )
        html_file.add_content(
            "<h1 style='color:white; font-size: 8em'>404</h1>"
        )
        html_file.add_content(
            "<h2 style='color:white;'> Something went wrong... </h2>"
        )
        html_file.end_div()
        html_file.end_div()
        html_file.close()

    def make_version_page(self):
        """Wrapper function for _make_version_page
        """
        pages = ["Version"]
        self.create_blank_html_pages(pages, stylesheets=pages)
        self._make_version_page(pages)

    def _make_version_page(self, pages):
        """Make a page to display the version information

        Parameters
        ----------
        pages: list
            list of pages that you wish to create
        """
        html_file = webpage.open_html(
            web_dir=self.webdir, base_url=self.base_url, html_page="Version"
        )
        html_file = self.setup_page(
            "Version", self.navbar["home"], title="Version Information"
        )
        html_file.make_banner(approximant="Version", key="Version")
        path = pesummary.__file__[:-12]
        with open(path + "/.version", 'r') as f:
            contents = f.read()
        for i in self.labels:
            contents = (
                "# {} version information\n\n{}_version={}\n\n".format(
                    i, i, self.file_versions[i]
                )
            ) + contents
        for i in self.labels:
            contents = "# %s version information\n\n%s_version = %s\n\n" % (
                i, i, self.file_versions[i]) + contents
        html_file.make_container()
        styles = html_file.make_code_block(language='shell', contents=contents)
        with open('{0:s}/css/Version.css'.format(self.webdir), 'w') as f:
            f.write(styles)
        html_file.end_container()
        html_file.make_footer(user=self.user, rundir=self.webdir)
        html_file.close()

    def make_logging_page(self):
        """Wrapper function for _make_logging_page
        """
        pages = ["Logging"]
        self.create_blank_html_pages(pages, stylesheets=pages)
        self._make_logging_page(pages)

    def _make_logging_page(self, pages):
        """Make a page to display the logging output from PESummary

        Parameters
        ----------
        pages: list
            list of pages that you wish to create
        """
        html_file = webpage.open_html(
            web_dir=self.webdir, base_url=self.base_url, html_page="Logging"
        )
        html_file = self.setup_page(
            "Logging", self.navbar["home"], title="Logger Information"
        )
        html_file.make_banner(approximant="Logging", key="Logging")
        path = pesummary.__file__[:-12]
        log_file = glob(".tmp/pesummary/*/pesummary.log")
        if len(log_file) == 0:
            log_file = ".tmp/pesummary/no_log_information.log"
            with open(log_file, "w") as f:
                f.writelines(["No log information stored"])
        else:
            log_file = log_file[0]
        with open(log_file, 'r') as f:
            contents = f.read()
        html_file.make_container()
        styles = html_file.make_code_block(language='shell', contents=contents)
        with open('{0:s}/css/Logging.css'.format(self.webdir), 'w') as f:
            f.write(styles)
        html_file.end_container()
        html_file.make_footer(user=self.user, rundir=self.webdir)
        html_file.close()

    def make_notes_page(self):
        """Wrapper function for _make_notes_page
        """
        pages = ["Notes"]
        self.create_blank_html_pages(pages, stylesheets=pages)
        self._make_notes_page(pages)

    def _make_notes_page(self, pages):
        """Make a page to display the custom notes

        Parameters
        ----------
        pages: list
            list of pages that you wish to create
        """
        html_file = webpage.open_html(
            web_dir=self.webdir, base_url=self.base_url, html_page="Notes"
        )
        html_file = self.setup_page(
            "Notes", self.navbar["home"], title="Notes"
        )
        html_file.make_banner(approximant="Notes", key="Notes")
        html_file.make_container()
        styles = html_file.make_code_block(
            language='shell', contents=self.notes
        )
        with open('{0:s}/css/Notes.css'.format(self.webdir), 'w') as f:
            f.write(styles)
        html_file.end_container()
        html_file.make_footer(user=self.user, rundir=self.webdir)
        html_file.close()

    def generate_specific_javascript(self):
        """Tailor the javascript to the specific situation.
        """
        path = self.webdir + "/js/grab.js"
        existing = open(path)
        existing = existing.readlines()
        ind = existing.index("    if ( param == approximant ) {\n")
        content = existing[:ind]
        for i in [list(j.keys())[0] for j in self._result_page_links()]:
            content.append("    if ( param == \"%s\" ) {\n" % (i))
            content.append("        approx = \"None\" \n")
            content.append("    }\n")
        for i in existing[ind + 1:]:
            content.append(i)
        new_file = open(path, "w")
        new_file.writelines(content)
        new_file.close()

    def default_categories(self):
        """Return the default categories
        """
        categories = {
            "A-D": {
                "accept": ["a", "A", "b", "B", "c", "C", "d", "D"], "reject": []
            },
            "E-F": {
                "accept": ["e", "E", "f", "F", "g", "G", "h", "H"], "reject": []
            },
            "I-L": {
                "accept": ["i", "I", "j", "J", "k", "K", "l", "L"], "reject": []
            },
            "M-P": {
                "accept": ["m", "M", "n", "N", "o", "O", "p", "P"], "reject": []
            },
            "Q-T": {
                "accept": ["q", "Q", "r", "R", "s", "S", "t", "T"], "reject": []
            },
            "U-X": {
                "accept": ["u", "U", "v", "V", "w", "W", "x", "X"], "reject": []
            },
            "Y-Z": {
                "accept": ["y", "Y", "z", "Z"], "reject": []
            }
        }
        return categories

    def default_popular_options(self):
        """Return a list of default options
        """
        return []

    def add_existing_data(self):
        """
        """
        from pesummary.utils.utils import _add_existing_data

        self = _add_existing_data(self)



#! /usr/bin/env python

# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from pesummary.utils.utils import logger
import subprocess


def command_line():
    """Generate an Argument Parser object to control the command line options
    """
    import argparse

    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("-w", "--webdir", dest="webdir",
                        help="make page and plots in DIR", metavar="DIR",
                        default="./")
    parser.add_argument("-s", "--samples", dest="samples",
                        help="Posterior samples hdf5 file",
                        default=None)
    return parser


def get_executable_path(executable):
    """Return the path to the executable

    Parameters
    ----------
    executable: str
        name of the executable
    """
    try:
        path = subprocess.check_output(["which", "%s" % (executable)])
        path = path.decode("utf-8")
    except Exception:
        path = None
    return path


def launch_lalinference(webdir, path_to_results_file, trigger_file=None):
    """Launch a subprocess to generate the lalinference pages

    Parameters
    ----------
    webdir: str
        path to the directory to store the output pages
    path_to_results_file: str
        path to the results file. Must be compatible with the LALInference
        pipeline
    trigger_file: str, optional
        path to an xml trigger file.
    """
    executable = "cbcBayesPostProc"
    executable_path = get_executable_path(executable)
    if executable_path is None:
        raise Exception(
            "'cbcbayespostproc' is not installed in your environment. failed "
            "to generate lalinference pages")

    webdir += "/lalinference"
    default_arguments = ["--skyres", "0.5", "--outpath", webdir,
                         path_to_results_file]

    if trigger_file is not None:
        default_arguments.append("--trig")
        default_arguments.append(trigger_file)

    cli = executable + " " + " ".join(default_arguments)
    logger.info("Running %s" % (cli))
    ess = subprocess.Popen(
        cli, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    ess.wait()


def launch_pesummary(webdir, path_to_results_file, trigger_file=None):
    """Launch a subprocess to generate the PESummary pages

    Parameters
    ----------
    webdir: str
        path to the directory to store the output pages
    path_to_results_file: str
        path to the results file. Must be compatible with the LALInference
        pipeline
    trigger_file: str, optional
        path to an xml trigger file.
    """
    executable = "summarypages"
    executable_path = get_executable_path(executable)

    if executable_path is None:
        raise Exception(
            "'summarypages' is not installed in your environment. failed "
            "to generate PESummary pages")

    webdir += "/pesummary"
    default_arguments = ["--webdir", webdir, "--samples", path_to_results_file,
                         "--approximant", "IMRPhenomPv2", "--gw",
                         "--labels", "pesummary", '--no_ligo_skymap']

    if trigger_file is not None:
        default_arguments.append("--trig_file")
        default_arguments.append(trigger_file)

    cli = executable + " " + " ".join(default_arguments)
    logger.info("Running %s" % (cli))
    ess = subprocess.Popen(
        cli, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    ess.wait()


def make_table_of_images(html_file, contents):
    """Make a table of images

    Parameters
    ----------
    html_file: pesummary.core.webpage.webpage.page
        open html file to write the information to
    contents: nd list
        list of images to put in the table
    """
    html_file.add_content("<div class='mx-auto d-block'>")
    html_file.add_content("<div class='row'>")
    for idx, j in enumerate(contents):
        html_file.add_content("<div class='column'>")
        html_file.add_content("<img src='%s' style='align-items:center; height:425px'>" % (j[0]))
        html_file.add_content("</div>")
    html_file.add_content("</div>")
    html_file.add_content("</div>")


def get_list_of_plots(webdir, lalinference=False, pesummary=False):
    """Get a list of plots generated by either lalinference or pesummary

    Parameters
    ----------
    webdir: str
        path to the directory to store the output pages
    lalinference: Bool
        if True, return the plots generated by LALInference. Default False
    pesummary: Bool
        if True, return the plots generated by PESummary. Default False
    """
    from glob import glob

    if lalinference:
        histogram_plots = sorted(
            glob(webdir + "/lalinference/1Dpdf/*.png"))
        params = [
            i.split("/")[-1].split(".png")[0] for i in histogram_plots]
    elif pesummary:
        histogram_plots = sorted(
            glob(webdir + "/pesummary/plots/*_1d_posterior*.png"))
        params = [
            i.split("/")[-1].split("_1d_posterior_")[1].split(".png")[0] for i
            in histogram_plots]
    return histogram_plots, params


def make_basic_comparison_html(webdir):
    """Make a basic comparison html page

    Parameters
    ----------
    webdir: str
        path to the directory to store the output pages
    """
    from pesummary.core.webpage import webpage
    from pesummary.gw.file.standard_names import standard_names
    from glob import glob

    lalinference_1d_histograms, lalinference_params = get_list_of_plots(
        webdir, lalinference=True)
    pesummary_1d_histograms, pesummary_params = get_list_of_plots(
        webdir, pesummary=True)
    names = [[standard_names[i], i] for i in lalinference_params if i in list(
             standard_names.keys())]
    extra = [i for i in lalinference_params if i not in list(
             standard_names.keys())]

    pages = ["home"]
    webpage.make_html(web_dir=webdir, pages=pages)
    html_file = webpage.open_html(
        web_dir=webdir, base_url="./", html_page="home")

    html_file.add_content("<link rel='stylesheet' href='./pesummary/css/font.css'>")
    html_file.make_div(_class="banner", _style="font-size: 48px")
    html_file.add_content("PESummary review")
    html_file.end_div()
    html_file.make_div(_class='paragraph')
    html_file.add_content("Here we show the comparison between the plots generated by "
                          "'cbcBayesPostProc' and 'PESummary'")
    html_file.end_div()

    html_file.make_div(_class="banner", _style="font-size:36px")
    html_file.add_content("1d histogram plot comparison")
    html_file.end_div()
    included_pesummary = []
    for num, i in enumerate(names):
        if i[0] in pesummary_params:
            included_pesummary.append(i[0])
            html_file.make_container()
            contents = [["./lalinference/1Dpdf/%s.png" % (i[1])],
                        ["./pesummary/plots/pesummary_1d_posterior_%s.png" % (i[0])]]
            make_table_of_images(html_file, contents)
            contents = [["./lalinference/1Dsamps/%s_acf.png" % (i[1])],
                        ["./pesummary/plots/pesummary_autocorrelation_%s.png" % (i[0])]]
            make_table_of_images(html_file, contents)
            contents = [["./lalinference/1Dsamps/%s_samps.png" % (i[1])],
                        ["./pesummary/plots/pesummary_sample_evolution_%s.png" % (i[0])]]
            make_table_of_images(html_file, contents)
            html_file.end_container()

    html_file.make_div(_class="banner", _style="font-size:28px")
    html_file.add_content("Plots only made my 'summarypages'")
    html_file.end_div()
    extra_pesummary = [i for i in pesummary_params if i not in included_pesummary]
    for num, i in enumerate(extra_pesummary):
        contents = [["./pesummary/plots/pesummary_1d_posterior_%s.png" % (i)]]
        html_file.make_container()
        make_table_of_images(html_file, contents)
        html_file.end_container()

    html_file.make_div(_class="banner", _style="font-size:28px")
    html_file.add_content("Plots only made my 'cbcBayesPostProc'")
    html_file.end_div()
    for num, i in enumerate(extra):
        contents = [["./lalinference/1Dpdf/%s.png" % (i)]]
        html_file.make_container()
        make_table_of_images(html_file, contents)
        html_file.end_container()

    html_file.make_div(_class="banner", _style="font-size:36px")
    html_file.add_content("Waveform plot comparison")
    html_file.end_div()
    html_file.make_container()
    contents = [["./lalinference/Waveform/WF_DetFrame.png"],
                ["./pesummary/plots/pesummary_waveform.png"],
                ["./pesummary/plots/pesummary_waveform_timedomain.png"]]
    make_table_of_images(html_file, contents)
    html_file.end_container()

    html_file.make_div(_class="banner", _style="font-size:36px")
    html_file.add_content("Skymap comparison")
    html_file.end_div()
    html_file.make_container()
    contents = [["./lalinference/skymap.png"],
                ["pesummary/plots/pesummary_skymap.png"]]
    make_table_of_images(html_file, contents)
    html_file.end_container()
    html_file.close()


def main():
    """Top level interface for `summaryreview`
    """
    import time

    parser = command_line()
    opts = parser.parse_args()
    logger.info("Starting to generate plots using 'cbcBayesPostProc'")
    t0 = time.time()
    launch_lalinference(opts.webdir, opts.samples)
    t1 = time.time() - t0
    logger.info("Starting to generate plots using 'summarypages'")
    t0 = time.time()
    launch_pesummary(opts.webdir, opts.samples)
    t2 = time.time() - t0
    logger.info("Making a basic comparison page")
    make_basic_comparison_html(opts.webdir)
    logger.info(
        "Different in runtime: 'cbcBayesPostProc' took {}s and 'summarypages' "
        "took {}s".format(t1, t2))


if __name__ == "__main__":
    main()



#! /usr/bin/env python

# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from pesummary.utils.utils import logger, gw_results_file
from pesummary.core.inputs import PostProcessing
from pesummary.gw.inputs import GWPostProcessing


class WebpageGeneration(object):
    """Wrapper class for _GWWebpageGeneration and _CoreWebpageGeneration

    Parameters
    ----------
    inputs: argparse.Namespace
        Namespace object containing the command line options
    colors: list, optional
        colors that you wish to use to distinguish different result files
    """
    def __init__(self, inputs, colors="default", gw=False):
        self.inputs = inputs
        self.colors = colors
        self.gw = gw
        self.generate_webpages()

    def generate_webpages(self):
        """Generate all plots for all result files passed
        """
        logger.info("Starting to generate webpages")
        if self.gw:
            object = _GWWebpageGeneration(self.inputs, colors=self.colors)
        else:
            object = _CoreWebpageGeneration(self.inputs, colors=self.colors)
        object.generate_webpages()
        logger.info("Finished generating webpages")


class _CoreWebpageGeneration(PostProcessing):
    """Class to generate all webpages for all result files with the Core module

    Parameters
    ----------
    inputs: argparse.Namespace
        Namespace object containing the command line options
    colors: list, optional
        colors that you wish to use to distinguish different result files
    """
    def __init__(self, inputs, colors="default"):
        from pesummary.core.webpage.main import _WebpageGeneration

        super(_CoreWebpageGeneration, self).__init__(inputs, colors)
        self.webpage_object = _WebpageGeneration(
            webdir=self.webdir, samples=self.samples, labels=self.labels,
            publication=self.publication, user=self.user, config=self.config,
            same_parameters=self.same_parameters, base_url=self.baseurl,
            file_versions=self.file_version, hdf5=self.hdf5, colors=self.colors,
            custom_plotting=self.custom_plotting,
            existing_labels=self.existing_labels,
            existing_config=self.existing_config,
            existing_file_version=self.existing_file_version,
            existing_injection_data=self.existing_injection_data,
            existing_samples=self.existing_samples,
            existing_metafile=self.existing,
            existing_file_kwargs=self.existing_file_kwargs,
            add_to_existing=self.add_to_existing, notes=self.notes
        )

    def generate_webpages(self):
        """Generate all webpages within the Core module
        """
        self.webpage_object.generate_webpages()


class _GWWebpageGeneration(GWPostProcessing):
    """Class to generate all webpages for all result files with the GW module

    Parameters
    ----------
    inputs: argparse.Namespace
        Namespace object containing the command line options
    colors: list, optional
        colors that you wish to use to distinguish different result files
    """
    def __init__(self, inputs, colors="default"):
        from pesummary.gw.webpage.main import _WebpageGeneration

        super(_GWWebpageGeneration, self).__init__(inputs, colors)
        key_data = self.grab_key_data_from_result_files()
        self.webpage_object = _WebpageGeneration(
            webdir=self.webdir, samples=self.samples, labels=self.labels,
            publication=self.publication, user=self.user, config=self.config,
            same_parameters=self.same_parameters, base_url=self.baseurl,
            file_versions=self.file_version, hdf5=self.hdf5, colors=self.colors,
            custom_plotting=self.custom_plotting, gracedb=self.gracedb,
            pepredicates_probs=self.pepredicates_probs,
            approximant=self.approximant, key_data=key_data,
            file_kwargs=self.file_kwargs, existing_labels=self.existing_labels,
            existing_config=self.existing_config,
            existing_file_version=self.existing_file_version,
            existing_injection_data=self.existing_injection_data,
            existing_samples=self.existing_samples,
            existing_metafile=self.existing,
            add_to_existing=self.add_to_existing,
            existing_file_kwargs=self.existing_file_kwargs,
            result_files=self.result_files, notes=self.notes,
            pastro_probs=self.pastro_probs
        )

    def generate_webpages(self):
        """Generate all webpages within the Core module
        """
        self.webpage_object.generate_webpages()


def main():
    """Top level interface for `summarypages`
    """
    from pesummary.core.command_line import command_line
    from pesummary.gw.command_line import insert_gwspecific_option_group
    from pesummary.utils import functions
    from .summaryplots import PlotGeneration

    parser = command_line()
    insert_gwspecific_option_group(parser)
    opts = parser.parse_args()
    func = functions(opts)
    args = func["input"](opts)
    PlotGeneration(args, gw=gw_results_file(opts))
    WebpageGeneration(args, gw=gw_results_file(opts))
    func["MetaFile"](args)
    func["FinishingTouches"](args)


if __name__ == "__main__":
    main()



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import subprocess
import os

from pesummary.utils.utils import logger
from pesummary.core.inputs import PostProcessing


class FinishingTouches(PostProcessing):
    """Class to handle the finishing touches

    Parameters
    ----------
    parser: argparser
        The parser containing the command line arguments
    """
    def __init__(self, inputs):
        super(FinishingTouches, self).__init__(inputs)
        self.send_email()
        logger.info("Complete. Webpages can be viewed at the following url "
                    "%s" % (self.baseurl + "/home.html"))

    def send_email(self, message=None):
        """Send notification email.
        """
        if self.email:
            logger.info("Sending email to %s" % (self.email))
            try:
                self._email_notify(message)
            except Exception as e:
                logger.info("Unable to send notification email because %s" % (
                    e))

    def _email_message(self, message=None):
        """Message that will be send in the email.
        """
        if not message:
            message = ("Hi %s,\n\nYour output page is ready on %s. You can "
                       "view the result at %s \n"
                       % (self.user, self.host, self.baseurl + "/home.html"))
        return message

    def _email_notify(self, message):
        """Subprocess to send the notification email.
        """
        subject = "Output page available at %s" % (self.host)
        message = self._email_message(message)
        cmd = 'echo -e "%s" | mail -s "%s" "%s"' % (
            message, subject, self.email)
        ess = subprocess.Popen(cmd, shell=True)
        ess.wait()

    def remove_tmp_directories(self):
        """Remove the temp directories created by PESummary
        """
        from pesummary.utils import utils

        utils.remove_tmp_directories()
